@@ -4,7 +4,7 @@
 #
 # Part of the Routino routing software.
 #
-# This file Copyright 2011-2014, 2018 Andrew M. Bishop
+# This file Copyright 2011-2014, 2018, 2023 Andrew M. Bishop
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -251,51 +251,53 @@ else
 
       print "$errors{$error} occurences; ";
 
-      if($errors{$error}>100)
-        {
-         print "too many to list individually.";
-        }
-      else
-        {
-         my @ids=sort({ return $a <=> $b } @{$errorids{$error}});
+      my @ids=sort({ return $a <=> $b } @{$errorids{$error}});
 
-         my $first=1;
+      my $first=1;
+      my $count=0;
 
-         foreach my $id (@ids)
+      foreach my $id (@ids)
+        {
+         if($first)
            {
-            if($first)
-              {
-               print "$errortypelabel{$errortypes{$error}}:\n";
-              }
-            else
-              {
-               print ",\n";
-              }
-
-            $first=0;
-
-            print "<a href=\"http://www.openstreetmap.org/browse/node/$id\">$id</a>" if($errortypes{$error} eq "N");
-            print "<a href=\"http://www.openstreetmap.org/browse/way/$id\">$id</a>" if($errortypes{$error} eq "W");
-            print "<a href=\"http://www.openstreetmap.org/browse/relation/$id\">$id</a>" if($errortypes{$error} eq "R");
-
-            if($errortypes{$error} eq "WN" || $errortypes{$error} eq "N2" || $errortypes{$error} eq "RN" || $errortypes{$error} eq "RW")
-              {
-               $id =~ m%\(([0-9]+) ([0-9]+)\)%;
-               print "(<a href=\"http://www.openstreetmap.org/browse/way/$1\">$1</a> <a href=\"http://www.openstreetmap.org/browse/node/$2\">$2</a>)" if($errortypes{$error} eq "WN");
-               print "(<a href=\"http://www.openstreetmap.org/browse/node/$1\">$1</a> <a href=\"http://www.openstreetmap.org/browse/node/$2\">$2</a>)" if($errortypes{$error} eq "N2");
-               print "(<a href=\"http://www.openstreetmap.org/browse/relation/$1\">$1</a> <a href=\"http://www.openstreetmap.org/browse/node/$2\">$2</a>)" if($errortypes{$error} eq "RN");
-               print "(<a href=\"http://www.openstreetmap.org/browse/relation/$1\">$1</a> <a href=\"http://www.openstreetmap.org/browse/way/$2\">$2</a>)" if($errortypes{$error} eq "RW");
-              }
-
-            if($errortypes{$error} eq "N2W")
-              {
-               $id =~ m%\(([0-9]+) ([0-9]+) ([0-9]+)\)%;
-               print "(<a href=\"http://www.openstreetmap.org/browse/node/$1\">$1</a> <a href=\"http://www.openstreetmap.org/browse/node/$2\">$2</a> <a href=\"http://www.openstreetmap.org/browse/way/$3\">$3</a>)" if($errortypes{$error} eq "N2W");
-              }
+            print "$errortypelabel{$errortypes{$error}}:\n";
+           }
+         else
+           {
+            print ",\n";
            }
 
-         print "\n";
+         $first=0;
+
+         print "<a href=\"http://www.openstreetmap.org/browse/node/$id\">$id</a>" if($errortypes{$error} eq "N");
+         print "<a href=\"http://www.openstreetmap.org/browse/way/$id\">$id</a>" if($errortypes{$error} eq "W");
+         print "<a href=\"http://www.openstreetmap.org/browse/relation/$id\">$id</a>" if($errortypes{$error} eq "R");
+
+         if($errortypes{$error} eq "WN" || $errortypes{$error} eq "N2" || $errortypes{$error} eq "RN" || $errortypes{$error} eq "RW")
+           {
+            $id =~ m%\(([0-9]+) ([0-9]+)\)%;
+            print "(<a href=\"http://www.openstreetmap.org/browse/way/$1\">$1</a> <a href=\"http://www.openstreetmap.org/browse/node/$2\">$2</a>)" if($errortypes{$error} eq "WN");
+            print "(<a href=\"http://www.openstreetmap.org/browse/node/$1\">$1</a> <a href=\"http://www.openstreetmap.org/browse/node/$2\">$2</a>)" if($errortypes{$error} eq "N2");
+            print "(<a href=\"http://www.openstreetmap.org/browse/relation/$1\">$1</a> <a href=\"http://www.openstreetmap.org/browse/node/$2\">$2</a>)" if($errortypes{$error} eq "RN");
+            print "(<a href=\"http://www.openstreetmap.org/browse/relation/$1\">$1</a> <a href=\"http://www.openstreetmap.org/browse/way/$2\">$2</a>)" if($errortypes{$error} eq "RW");
+           }
+
+         if($errortypes{$error} eq "N2W")
+           {
+            $id =~ m%\(([0-9]+) ([0-9]+) ([0-9]+)\)%;
+            print "(<a href=\"http://www.openstreetmap.org/browse/node/$1\">$1</a> <a href=\"http://www.openstreetmap.org/browse/node/$2\">$2</a> <a href=\"http://www.openstreetmap.org/browse/way/$3\">$3</a>)" if($errortypes{$error} eq "N2W");
+           }
+
+         $count++;
+
+         if($count>=100)
+           {
+            print " [first 100 listed].";
+            last;
+           }
         }
+
+      print "\n";
      }
 
    print "\n".
