@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2015, 2023 Andrew M. Bishop
+ This file Copyright 2008-2015, 2023, 2025 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -211,17 +211,17 @@ int main(int argc,char** argv)
     if((p=strstr(filename,".pbf")) && !strcmp(p,".pbf"))
       {
        if(ParsePBFFile(fd,OSMNodes,OSMWays,OSMRelations))
-          exit(EXIT_FAILURE);
+          exit(EXIT_FAILURE); /* no message because one printed in parser function */
       }
     else if((p=strstr(filename,".o5m")) && !strcmp(p,".o5m"))
       {
        if(ParseO5MFile(fd,OSMNodes,OSMWays,OSMRelations))
-          exit(EXIT_FAILURE);
+          exit(EXIT_FAILURE); /* no message because one printed in parser function */
       }
     else
       {
        if(ParseOSMFile(fd,OSMNodes,OSMWays,OSMRelations))
-          exit(EXIT_FAILURE);
+          exit(EXIT_FAILURE); /* no message because one printed in parser function */
       }
 
     CloseFile(fd);
