@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2015 Andrew M. Bishop
+ This file Copyright 2008-2015, 2023 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -52,6 +52,12 @@ static int         relation_nways;
 static relation_t *relation_relations;
 static int         relation_nrelations;
 
+/* Local parsing functions */
+
+static void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longitude,int mode);
+static void ProcessWayTags(TagList *tags,int64_t way_id,int mode);
+static void ProcessRelationTags(TagList *tags,int64_t relation_id,int mode);
+
 
 /*++++++++++++++++++++++++++++++++++++++
   Initialise the OSM parser by initialising the local variables.
@@ -181,6 +187,73 @@ void AddRelationRefs(int64_t node_id,int64_t way_id,int64_t relation_id,const ch
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Add a node after processing the tags associated with it.
+
+  int64_t node_id The id of the node.
+
+  double latitude The latitude of the node.
+
+  double longitude The longitude of the node.
+
+  int mode The mode of operation to take (create, modify, delete).
+
+  TagList *raw_tags The accumulated set of tags for the node.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void AddNode(int64_t node_id,double latitude,double longitude,int mode,TagList *raw_tags)
+{
+ TagList *processed_tags=ApplyNodeTaggingRules(raw_tags,node_id);
+
+ ProcessNodeTags(processed_tags,node_id,latitude,longitude,mode);
+
+ DeleteTagList(raw_tags);
+ DeleteTagList(processed_tags);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Add a way after processing the tags associated with it.
+
+  int64_t way_id The id of the way.
+
+  int mode The mode of operation to take (create, modify, delete).
+
+  TagList *raw_tags The accumulated set of tags for the node.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void AddWay(int64_t way_id,int mode,TagList *raw_tags)
+{
+ TagList *processed_tags=ApplyWayTaggingRules(raw_tags,way_id);
+
+ ProcessWayTags(processed_tags,way_id,mode);
+
+ DeleteTagList(raw_tags);
+ DeleteTagList(processed_tags);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Add a relation after processing the tags associated with it.
+
+  int64_t relation_id The id of the relation.
+
+  int mode The mode of operation to take (create, modify, delete).
+
+  TagList *raw_tags The accumulated set of tags for the node.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void AddRelation(int64_t relation_id,int mode,TagList *raw_tags)
+{
+ TagList *processed_tags=ApplyRelationTaggingRules(raw_tags,relation_id);
+
+ ProcessRelationTags(processed_tags,relation_id,mode);
+
+ DeleteTagList(raw_tags);
+ DeleteTagList(processed_tags);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Process the tags associated with a node.
 
@@ -195,7 +268,7 @@ void AddRelationRefs(int64_t node_id,int64_t way_id,int64_t relation_id,const ch
   int mode The mode of operation to take (create, modify, delete).
   ++++++++++++++++++++++++++++++++++++++*/
 
-void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longitude,int mode)
+static void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longitude,int mode)
 {
  node_t id;
  int i;
@@ -234,7 +307,7 @@ void ProcessNodeTags(TagList *tags,int64_t node_id,double latitude,double longit
   int mode The mode of operation to take (create, modify, delete).
   ++++++++++++++++++++++++++++++++++++++*/
 
-void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
+static void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
 {
  Way way={0};
  way_t id;
@@ -274,7 +347,7 @@ void ProcessWayTags(TagList *tags,int64_t way_id,int mode)
   int mode The mode of operation to take (create, modify, delete).
   ++++++++++++++++++++++++++++++++++++++*/
 
-void ProcessRelationTags(TagList *tags,int64_t relation_id,int mode)
+static void ProcessRelationTags(TagList *tags,int64_t relation_id,int mode)
 {
  relation_t id;
  int i;
