@@ -2,7 +2,7 @@
 #
 # Part of the Routino routing software.
 #
-# This file Copyright 2018, 2022 Andrew M. Bishop
+# This file Copyright 2018, 2022, 2024 Andrew M. Bishop
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -24,6 +24,7 @@ from setuptools import setup, Extension
 
 routino_router = Extension('routino._router',
                            sources = ['src/_router.c'],
+                           depends = ['../src/routino.h'],
                            include_dirs = ['../src'],
                            library_dirs = ['../src'],
                            libraries = ['routino'])
@@ -40,6 +41,7 @@ for file in os.listdir('../src'):
 
 routino_database = Extension('routino._database',
                              sources = ['src/_database.cc', 'src/database.cc'],
+                             depends = ['../src/types.h', '../src/nodes.h', '../src/segments.h', '../src/ways.h', '../src/relations.h', '../src/routino.h'],
                              include_dirs = ['../src'],
                              extra_objects = lib_files,
                              library_dirs = ['../src'])
