@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2015 Andrew M. Bishop
+ This file Copyright 2008-2015, 2024, 2025 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -63,10 +63,10 @@
 /*+ A structure to contain the list of memory mapped files. +*/
 struct mmapinfo
 {
- const char  *filename;         /*+ The name of the file (the index of the list). +*/
-       int    fd;               /*+ The file descriptor used when it was opened. +*/
-       char  *address;          /*+ The address the file was mapped to. +*/
-       size_t length;           /*+ The length of the file. +*/
+ char  *filename;         /*+ The name of the file (the index of the list). +*/
+ int    fd;               /*+ The file descriptor used when it was opened. +*/
+ char  *address;          /*+ The address the file was mapped to. +*/
+ size_t length;           /*+ The length of the file. +*/
 };
 
 /*+ The list of memory mapped files. +*/
@@ -81,10 +81,14 @@ static int nmappedfiles=0;
 /*+ A structure to contain the list of file buffers. +*/
 struct filebuffer
 {
- char   buffer[BUFFLEN];        /*+ The data buffer. +*/
- size_t pointer;                /*+ The read/write pointer for the file buffer. +*/
- size_t length;                 /*+ The read pointer for the file buffer. +*/
- int    reading;                /*+ A flag to indicate if the file is for reading. +*/
+ char   buffer[BUFFLEN];   /*+ The data buffer. +*/
+ size_t pointer;           /*+ The read/write pointer for the file buffer. +*/
+ size_t length;            /*+ The read pointer for the file buffer. +*/
+ int    reading;           /*+ A flag to indicate if the file is for reading. +*/
+ char  *filename;          /*+ The name of the file. +*/
+#if defined(_MSC_VER) || defined(__MINGW32__)
+ int    delete;            /*+ Set to non-zero value if the file is to be deleted when closed. +*/
+#endif
 };
 
 /*+ The list of file buffers. +*/
@@ -94,33 +98,9 @@ static struct filebuffer **filebuffers=NULL;
 static int nfilebuffers=0;
 
 
-#if defined(_MSC_VER) || defined(__MINGW32__)
-
-/*+ A structure to contain the list of opened files to record which are to be deleted when closed. +*/
-struct openedfile
-{
- const char *filename;          /*+ The name of the file. +*/
-       int   delete;            /*+ Set to non-zero value if the file is to be deleted when closed. +*/
-};
-
-/*+ The list of opened files. +*/
-static struct openedfile **openedfiles=NULL;
-
-/*+ The number of allocated opened file buffer pointers. +*/
-static int nopenedfiles=0;
-
-#endif
-
-
 /* Local functions */
 
-static void CreateFileBuffer(int fd,int read_write);
-
-#if defined(_MSC_VER) || defined(__MINGW32__)
-
-static void CreateOpenedFile(int fd,const char *filename);
-
-#endif
+static void CreateFileBuffer(int fd,int read_write,const char *filename);
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -173,8 +153,7 @@ void *MapFile(const char *filename)
 #ifdef LIBROUTINO
     return(NULL);
 #else
-    fprintf(stderr,"Cannot open file '%s' for reading [%s].\n",filename,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot open file '%s' for reading [%s].",filename,strerror(errno)));
 #endif
    }
 
@@ -185,8 +164,7 @@ void *MapFile(const char *filename)
 #ifdef LIBROUTINO
     return(NULL);
 #else
-    fprintf(stderr,"Cannot stat file '%s' [%s].\n",filename,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot stat file '%s' [%s].",filename,strerror(errno)));
 #endif
    }
 
@@ -203,8 +181,7 @@ void *MapFile(const char *filename)
 #ifdef LIBROUTINO
     return(NULL);
 #else
-    fprintf(stderr,"Cannot mmap file '%s' for reading [%s].\n",filename,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot mmap file '%s' for reading [%s].",filename,strerror(errno)));
 #endif
    }
 
@@ -216,7 +193,7 @@ void *MapFile(const char *filename)
 
  mappedfiles=(struct mmapinfo*)realloc((void*)mappedfiles,(nmappedfiles+1)*sizeof(struct mmapinfo));
 
- mappedfiles[nmappedfiles].filename=filename;
+ mappedfiles[nmappedfiles].filename=strcpy(malloc(strlen(filename)+1),filename);
  mappedfiles[nmappedfiles].fd=fd;
  mappedfiles[nmappedfiles].address=address;
  mappedfiles[nmappedfiles].length=size;
@@ -255,8 +232,7 @@ void *MapFileWriteable(const char *filename)
 #ifdef LIBROUTINO
     return(NULL);
 #else
-    fprintf(stderr,"Cannot open file '%s' for reading and writing [%s].\n",filename,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot open file '%s' for reading and writing [%s].",filename,strerror(errno)));
 #endif
    }
 
@@ -267,8 +243,7 @@ void *MapFileWriteable(const char *filename)
 #ifdef LIBROUTINO
     return(NULL);
 #else
-    fprintf(stderr,"Cannot stat file '%s' [%s].\n",filename,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot stat file '%s' [%s].",filename,strerror(errno)));
 #endif
    }
 
@@ -285,8 +260,7 @@ void *MapFileWriteable(const char *filename)
 #ifdef LIBROUTINO
     return(NULL);
 #else
-    fprintf(stderr,"Cannot mmap file '%s' for reading and writing [%s].\n",filename,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot mmap file '%s' for reading and writing [%s].",filename,strerror(errno)));
 #endif
    }
 
@@ -298,7 +272,7 @@ void *MapFileWriteable(const char *filename)
 
  mappedfiles=(struct mmapinfo*)realloc((void*)mappedfiles,(nmappedfiles+1)*sizeof(struct mmapinfo));
 
- mappedfiles[nmappedfiles].filename=filename;
+ mappedfiles[nmappedfiles].filename=strcpy(malloc(strlen(filename)+1),filename);
  mappedfiles[nmappedfiles].fd=fd;
  mappedfiles[nmappedfiles].address=address;
  mappedfiles[nmappedfiles].length=size;
@@ -330,8 +304,7 @@ void *UnmapFile(const void *address)
 #ifdef LIBROUTINO
     return(NULL);
 #else
-    fprintf(stderr,"The data at address %p was not mapped using MapFile().\n",address);
-    exit(EXIT_FAILURE);
+    logassert_format(0,("The data at address %p was not mapped using MapFile() - report a bug.",address));
 #endif
    }
 
@@ -339,6 +312,8 @@ void *UnmapFile(const void *address)
 
  close(mappedfiles[i].fd);
 
+ free(mappedfiles[i].filename);
+
  /* Unmap the file */
 
  munmap(mappedfiles[i].address,mappedfiles[i].length);
@@ -383,13 +358,10 @@ int SlimMapFile(const char *filename)
 #ifdef LIBROUTINO
     return(-1);
 #else
-    fprintf(stderr,"Cannot open file '%s' for reading [%s].\n",filename,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot open file '%s' for reading [%s].",filename,strerror(errno)));
 #endif
    }
 
- CreateFileBuffer(fd,0);
-
  return(fd);
 }
 
@@ -419,13 +391,10 @@ int SlimMapFileWriteable(const char *filename)
 #ifdef LIBROUTINO
     return(-1);
 #else
-    fprintf(stderr,"Cannot open file '%s' for reading and writing [%s].\n",filename,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot open file '%s' for reading and writing [%s].",filename,strerror(errno)));
 #endif
    }
 
- CreateFileBuffer(fd,0);
-
  return(fd);
 }
 
@@ -471,16 +440,11 @@ int OpenFileBufferedNew(const char *filename)
 #ifdef LIBROUTINO
     return(-1);
 #else
-    fprintf(stderr,"Cannot open file '%s' for writing [%s].\n",filename,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot open file '%s' for writing [%s].",filename,strerror(errno)));
 #endif
    }
 
- CreateFileBuffer(fd,-1);
-
-#if defined(_MSC_VER) || defined(__MINGW32__)
- CreateOpenedFile(fd,filename);
-#endif
+ CreateFileBuffer(fd,-1,filename);
 
  return(fd);
 }
@@ -511,16 +475,11 @@ int OpenFileBufferedAppend(const char *filename)
 #ifdef LIBROUTINO
     return(-1);
 #else
-    fprintf(stderr,"Cannot open file '%s' for appending [%s].\n",filename,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot open file '%s' for appending [%s].",filename,strerror(errno)));
 #endif
    }
 
- CreateFileBuffer(fd,-1);
-
-#if defined(_MSC_VER) || defined(__MINGW32__)
- CreateOpenedFile(fd,filename);
-#endif
+ CreateFileBuffer(fd,-1,filename);
 
  return(fd);
 }
@@ -551,16 +510,11 @@ int ReOpenFileBuffered(const char *filename)
 #ifdef LIBROUTINO
     return(-1);
 #else
-    fprintf(stderr,"Cannot open file '%s' for reading [%s].\n",filename,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot open file '%s' for reading [%s].",filename,strerror(errno)));
 #endif
    }
 
- CreateFileBuffer(fd,1);
-
-#if defined(_MSC_VER) || defined(__MINGW32__)
- CreateOpenedFile(fd,filename);
-#endif
+ CreateFileBuffer(fd,1,filename);
 
  return(fd);
 }
@@ -592,13 +546,13 @@ int ReplaceFileBuffered(const char *filename,int *oldfd)
 
  *oldfd=ReOpenFileBuffered(filename2);
  
- DeleteFile(filename2);
+ DeleteFileBuffered(filename2);
 
 #else
 
  *oldfd=ReOpenFileBuffered(filename);
  
- DeleteFile(filename);
+ DeleteFileBuffered(filename);
 
 #endif
 
@@ -635,7 +589,13 @@ int WriteFileBuffered(int fd,const void *address,size_t length)
  if((filebuffers[fd]->pointer+length)>BUFFLEN)
    {
     if(write(fd,filebuffers[fd]->buffer,filebuffers[fd]->pointer)!=(ssize_t)filebuffers[fd]->pointer)
+      {
+#ifdef LIBROUTINO
        return(-1);
+#else
+       logassert_format(0,("Cannot write to file '%s' [%s].",filebuffers[fd]->filename,strerror(errno)));
+#endif
+      }
 
     filebuffers[fd]->pointer=0;
    }
@@ -643,7 +603,13 @@ int WriteFileBuffered(int fd,const void *address,size_t length)
  if(length>=BUFFLEN)
    {
     if(write(fd,address,length)!=(ssize_t)length)
+      {
+#ifdef LIBROUTINO
        return(-1);
+#else
+       logassert_format(0,("Cannot write to file '%s' [%s].",filebuffers[fd]->filename,strerror(errno)));
+#endif
+      }
 
     return(0);
    }
@@ -695,7 +661,13 @@ int ReadFileBuffered(int fd,void *address,size_t length)
  if(length>=BUFFLEN)
    {
     if(read(fd,address,length)!=(ssize_t)length)
+      {
+#ifdef LIBROUTINO
        return(-1);
+#else
+       logassert_format(0,("Cannot read from file '%s' [%s].",filebuffers[fd]->filename,strerror(errno)));
+#endif
+      }
 
     return(0);
    }
@@ -704,7 +676,15 @@ int ReadFileBuffered(int fd,void *address,size_t length)
    {
     ssize_t len=read(fd,filebuffers[fd]->buffer,BUFFLEN);
 
-    if(len<=0)
+    if(len<0)
+      {
+#ifdef LIBROUTINO
+       return(-1);
+#else
+       logassert_format(0,("Cannot read from file '%s' [%s].",filebuffers[fd]->filename,strerror(errno)));
+#endif
+      }
+    else if(len==0)
        return(-1);
 
     filebuffers[fd]->length=len;
@@ -744,13 +724,25 @@ int SeekFileBuffered(int fd,offset_t position)
 
  if(!filebuffers[fd]->reading)
     if(write(fd,filebuffers[fd]->buffer,filebuffers[fd]->pointer)!=(ssize_t)filebuffers[fd]->pointer)
+      {
+#ifdef LIBROUTINO
        return(-1);
+#else
+       logassert_format(0,("Cannot write to file '%s' [%s].",filebuffers[fd]->filename,strerror(errno)));
+#endif
+      }
 
  filebuffers[fd]->pointer=0;
  filebuffers[fd]->length=0;
 
  if(lseek(fd,position,SEEK_SET)!=position)
+   {
+#ifdef LIBROUTINO
     return(-1);
+#else
+    logassert_format(0,("Cannot seek in file '%s' [%s].",filebuffers[fd]->filename,strerror(errno)));
+#endif
+   }
 
  return(0);
 }
@@ -786,7 +778,13 @@ int SkipFileBuffered(int fd,offset_t skip)
     filebuffers[fd]->length=0;
 
     if(lseek(fd,skip,SEEK_CUR)==-1)
+      {
+#ifdef LIBROUTINO
        return(-1);
+#else
+       logassert_format(0,("Cannot seek in file '%s' [%s].",filebuffers[fd]->filename,strerror(errno)));
+#endif
+      }
    }
  else
     filebuffers[fd]->pointer+=skip;
@@ -795,6 +793,115 @@ int SkipFileBuffered(int fd,offset_t skip)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Close a file on disk (and flush the buffer).
+
+  int CloseFileBuffered returns -1 (for similarity to the *OpenFileBuffered* functions).
+
+  int fd The file descriptor to close.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int CloseFileBuffered(int fd)
+{
+#ifndef LIBROUTINO
+ logassert(fd<nfilebuffers && filebuffers[fd],"File descriptor has no buffer - report a bug");
+#endif
+
+ if(!filebuffers[fd]->reading)
+    if(write(fd,filebuffers[fd]->buffer,filebuffers[fd]->pointer)!=(ssize_t)filebuffers[fd]->pointer)
+      {
+#ifdef LIBROUTINO
+       return(-1);
+#else
+       logassert_format(0,("Cannot write to file '%s' [%s].",filebuffers[fd]->filename,strerror(errno)));
+#endif
+      }
+
+ close(fd);
+
+ /* Delete the file if Windows */
+
+#if defined(_MSC_VER) || defined(__MINGW32__)
+ if(filebuffers[fd]->delete)
+    unlink(filebuffers[fd]->filename);
+#endif
+
+ /* Free the file buffer */
+
+ free(filebuffers[fd]->filename);
+ free(filebuffers[fd]);
+ filebuffers[fd]=NULL;
+
+ return(-1);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Create a file buffer.
+
+  int fd The file descriptor.
+
+  int read_write A flag set to 1 for reading, -1 for writing and 0 for unbuffered.
+
+  const char *filename The name of the file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void CreateFileBuffer(int fd,int read_write,const char *filename)
+{
+ if(nfilebuffers<=fd)
+   {
+    int i;
+
+    filebuffers=(struct filebuffer**)realloc((void*)filebuffers,(fd+1)*sizeof(struct filebuffer*));
+
+    for(i=nfilebuffers;i<=fd;i++)
+       filebuffers[i]=NULL;
+
+    nfilebuffers=fd+1;
+   }
+
+ filebuffers[fd]=(struct filebuffer*)calloc(1,sizeof(struct filebuffer));
+
+ filebuffers[fd]->reading=(read_write==1);
+
+ filebuffers[fd]->filename=strcpy(malloc(strlen(filename)+1),filename);
+
+#if defined(_MSC_VER) || defined(__MINGW32__)
+ filebuffers[fd]->delete=0;
+#endif
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Delete a file from disk.
+
+  int DeleteFileBuffered Returns 0 if OK.
+
+  const char *filename The name of the file to delete.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int DeleteFileBuffered(const char *filename)
+{
+ int fd;
+
+ for(fd=0;fd<nfilebuffers;fd++)
+    if(filebuffers[fd] && !strcmp(filebuffers[fd]->filename,filename))
+       break;
+
+#ifndef LIBROUTINO
+ logassert_format(fd<nfilebuffers && filebuffers[fd],("File '%s' is not buffered - report a bug.",filebuffers[fd]->filename));
+#endif
+
+#if defined(_MSC_VER) || defined(__MINGW32__)
+ filebuffers[fd]->delete=1;
+#else
+ unlink(filename);
+#endif
+
+ return(0);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Get the size of a file.
 
@@ -812,8 +919,7 @@ offset_t SizeFile(const char *filename)
 #ifdef LIBROUTINO
     return(-1);
 #else
-    fprintf(stderr,"Cannot stat file '%s' [%s].\n",filename,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot stat file '%s' [%s].",filename,strerror(errno)));
 #endif
    }
 
@@ -838,8 +944,7 @@ offset_t SizeFileFD(int fd)
 #ifdef LIBROUTINO
     return(-1);
 #else
-    fprintf(stderr,"Cannot stat file descriptor '%d' [%s].\n",fd,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot stat file descriptor '%d' [%s].",fd,strerror(errno)));
 #endif
    }
 
@@ -866,47 +971,6 @@ int ExistsFile(const char *filename)
 }
 
 
-/*++++++++++++++++++++++++++++++++++++++
-  Close a file on disk (and flush the buffer).
-
-  int CloseFileBuffered returns -1 (for similarity to the *OpenFileBuffered* functions).
-
-  int fd The file descriptor to close.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-int CloseFileBuffered(int fd)
-{
-#ifndef LIBROUTINO
- logassert(fd<nfilebuffers && filebuffers[fd],"File descriptor has no buffer - report a bug");
-#endif
-
- if(!filebuffers[fd]->reading)
-    if(write(fd,filebuffers[fd]->buffer,filebuffers[fd]->pointer)!=(ssize_t)filebuffers[fd]->pointer)
-       return(-1);
-
- close(fd);
-
- free(filebuffers[fd]);
- filebuffers[fd]=NULL;
-
-#if defined(_MSC_VER) || defined(__MINGW32__)
-
-#ifndef LIBROUTINO
- logassert(fd<nopenedfiles && openedfiles[fd],"File descriptor has no record of opening - report a bug");
-#endif
-
- if(openedfiles[fd]->delete)
-    unlink(openedfiles[fd]->filename);
-
- free(openedfiles[fd]);
- openedfiles[fd]=NULL;
-
-#endif
-
- return(-1);
-}
-
-
 /*++++++++++++++++++++++++++++++++++++++
   Open an existing file on disk for reading (in a simple mode).
 
@@ -932,15 +996,10 @@ int OpenFile(const char *filename)
 #ifdef LIBROUTINO
     return(-1);
 #else
-    fprintf(stderr,"Cannot open file '%s' for reading [%s].\n",filename,strerror(errno));
-    exit(EXIT_FAILURE);
+    logassert_format(0,("Cannot open file '%s' for reading [%s].",filename,strerror(errno)));
 #endif
    }
 
-#if defined(_MSC_VER) || defined(__MINGW32__)
- CreateOpenedFile(fd,filename);
-#endif
-
  return(fd);
 }
 
@@ -953,26 +1012,16 @@ int OpenFile(const char *filename)
 
 void CloseFile(int fd)
 {
- close(fd);
-
-#if defined(_MSC_VER) || defined(__MINGW32__)
-
 #ifndef LIBROUTINO
- logassert(fd<nopenedfiles && openedfiles[fd],"File descriptor has no record of opening - report a bug");
+ logassert_format(fd>=nfilebuffers || !filebuffers[fd],("File '%s' is buffered - report a bug.",filebuffers[fd]->filename));
 #endif
 
- if(openedfiles[fd]->delete)
-    unlink(openedfiles[fd]->filename);
-
- free(openedfiles[fd]);
- openedfiles[fd]=NULL;
-
-#endif
+ close(fd);
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Delete a file from disk.
+  Delete a file from disk (that was opened in simple mode).
 
   int DeleteFile Returns 0 if OK.
 
@@ -981,17 +1030,14 @@ void CloseFile(int fd)
 
 int DeleteFile(const char *filename)
 {
-#if defined(_MSC_VER) || defined(__MINGW32__)
-
  int fd;
 
- for(fd=0;fd<nopenedfiles;fd++)
-    if(openedfiles[fd] && !strcmp(openedfiles[fd]->filename,filename))
-      {
-       openedfiles[fd]->delete=1;
-       return(0);
-      }
+ for(fd=0;fd<nfilebuffers;fd++)
+    if(filebuffers[fd] && !strcmp(filebuffers[fd]->filename,filename))
+       break;
 
+#ifndef LIBROUTINO
+ logassert_format(fd==nfilebuffers || !filebuffers[fd],("File '%s' is buffered - report a bug.",filebuffers[fd]->filename));
 #endif
 
  unlink(filename);
@@ -1016,67 +1062,3 @@ int RenameFile(const char *oldfilename,const char *newfilename)
 
  return(0);
 }
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Create a file buffer.
-
-  int fd The file descriptor.
-
-  int read_write A flag set to 1 for reading, -1 for writing and 0 for unbuffered.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static void CreateFileBuffer(int fd,int read_write)
-{
- if(nfilebuffers<=fd)
-   {
-    int i;
-
-    filebuffers=(struct filebuffer**)realloc((void*)filebuffers,(fd+1)*sizeof(struct filebuffer*));
-
-    for(i=nfilebuffers;i<=fd;i++)
-       filebuffers[i]=NULL;
-
-    nfilebuffers=fd+1;
-   }
-
- if(read_write)
-   {
-    filebuffers[fd]=(struct filebuffer*)calloc(sizeof(struct filebuffer),1);
-
-    filebuffers[fd]->reading=(read_write==1);
-   }
-}
-
-
-#if defined(_MSC_VER) || defined(__MINGW32__)
-
-/*++++++++++++++++++++++++++++++++++++++
-  Create an opened file record.
-
-  int fd The file descriptor.
-
-  const char *filename The name of the file.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static void CreateOpenedFile(int fd,const char *filename)
-{
- if(nopenedfiles<=fd)
-   {
-    int i;
-
-    openedfiles=(struct openedfile**)realloc((void*)openedfiles,(fd+1)*sizeof(struct openedfile*));
-
-    for(i=nopenedfiles;i<=fd;i++)
-       openedfiles[i]=NULL;
-
-    nopenedfiles=fd+1;
-   }
-
- openedfiles[fd]=(struct openedfile*)calloc(sizeof(struct openedfile),1);
-
- openedfiles[fd]->filename=strcpy(malloc(strlen(filename)+1),filename);
- openedfiles[fd]->delete=0;
-}
-
-#endif
