@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2015, 2019 Andrew M. Bishop
+ This file Copyright 2008-2015, 2019, 2025 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -65,6 +65,8 @@ typedef int64_t offset_t;
 
 char *FileName(const char *dirname,const char *prefix, const char *name);
 
+/* Functions in files.c for mapped files */
+
 void *MapFile(const char *filename);
 void *MapFileWriteable(const char *filename);
 
@@ -75,6 +77,8 @@ int SlimMapFileWriteable(const char *filename);
 
 int SlimUnmapFile(int fd);
 
+/* Functions in files.c for buffered files */
+
 int OpenFileBufferedNew(const char *filename);
 int OpenFileBufferedAppend(const char *filename);
 
@@ -90,16 +94,20 @@ int SkipFileBuffered(int fd,offset_t skip);
 
 int CloseFileBuffered(int fd);
 
+int DeleteFileBuffered(const char *filename);
+
+/* Functions in files.c for simple files */
+
 int OpenFile(const char *filename);
 
 void CloseFile(int fd);
 
+int DeleteFile(const char *filename);
+
 offset_t SizeFile(const char *filename);
 offset_t SizeFileFD(int fd);
 int ExistsFile(const char *filename);
 
-int DeleteFile(const char *filename);
-
 int RenameFile(const char *oldfilename,const char *newfilename);
 
 /* Functions in files.h */
