@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2013, 2015 Andrew M. Bishop
+ This file Copyright 2013, 2015, 2025 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -75,11 +75,7 @@ void open_errorlog(const char *filename,int append,int bin)
  errorlogfile=fopen(errorlogfilename,append?"a" :"w" );
 #endif
 
- if(!errorlogfile)
-   {
-    fprintf(stderr,"Cannot open file '%s' for writing [%s].\n",errorlogfilename,strerror(errno));
-    exit(EXIT_FAILURE);
-   }
+ logassert_format(errorlogfile,("Cannot open file '%s' for writing [%s].",errorlogfilename,strerror(errno)));
 
  /* Binary log file */
 
