@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2015, 2017, 2019 Andrew M. Bishop
+ This file Copyright 2008-2015, 2017, 2019, 2025 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -106,16 +106,21 @@ relation_t logerror_relation(relation_t id);
 
 /* Runtime fatal error assertion in logging.c */
 
-#define logassert(xx,yy) do { if(!(xx)) _logassert(yy,__FILE__,__LINE__); } while(0)
+#define logassert(xx,yy)        do { if(!(xx)) _logassert(                  yy,__FILE__,__LINE__); } while(0)
+#define logassert_format(xx,yy) do { if(!(xx)) _logassert(_logassert_format yy,__FILE__,__LINE__); } while(0)
 
 #ifdef __GNUC__
 
 void _logassert(const char *message,const char *file,int line) __attribute__ ((noreturn));
 
+const char *_logassert_format(const char *format, ...) __attribute__ ((format (printf, 1, 2)));
+
 #else
 
 void _logassert(const char *message,const char *file,int line);
 
+const char *_logassert_format(const char *format, ...);
+
 #endif
 
 
