@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2012-2015, 2017, 2019 Andrew M. Bishop
+ This file Copyright 2012-2015, 2017, 2019, 2023 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -489,7 +489,7 @@ static void process_node(void)
  int64_t delta_id;
  int32_t delta_lat;
  int32_t delta_lon;
- TagList *tags=NULL,*result=NULL;
+ TagList *tags=NULL;
  int mode=mode_change;
 
  delta_id=o5m_sint64(&buffer_ptr);
@@ -532,12 +532,7 @@ static void process_node(void)
     AppendTag(tags,(char*)key,(char*)val);
    }
 
- result=ApplyNodeTaggingRules(tags,id);
-
- ProcessNodeTags(result,id,O5M_LATITUDE(lat),O5M_LONGITUDE(lon),mode);
-
- DeleteTagList(tags);
- DeleteTagList(result);
+ AddNode(id,O5M_LATITUDE(lat),O5M_LONGITUDE(lon),mode,tags);
 }
 
 
@@ -548,7 +543,7 @@ static void process_node(void)
 static void process_way(void)
 {
  int64_t delta_id;
- TagList *tags=NULL,*result=NULL;
+ TagList *tags=NULL;
  int mode=mode_change;
  unsigned char *refs=NULL,*refs_end=NULL;
 
@@ -611,12 +606,7 @@ static void process_way(void)
     AppendTag(tags,(char*)key,(char*)val);
    }
 
- result=ApplyWayTaggingRules(tags,id);
-
- ProcessWayTags(result,id,mode);
-
- DeleteTagList(tags);
- DeleteTagList(result);
+ AddWay(id,mode,tags);
 }
 
 
@@ -627,7 +617,7 @@ static void process_way(void)
 static void process_relation()
 {
  int64_t delta_id;
- TagList *tags=NULL,*result=NULL;
+ TagList *tags=NULL;
  int mode=mode_change;
  unsigned char *refs=NULL,*refs_end=NULL;
 
@@ -709,12 +699,7 @@ static void process_relation()
     AppendTag(tags,(char*)key,(char*)val);
    }
 
- result=ApplyRelationTaggingRules(tags,id);
-
- ProcessRelationTags(result,id,mode);
-
- DeleteTagList(tags);
- DeleteTagList(result);
+ AddRelation(id,mode,tags);
 }
 
 
