@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2012-2015 Andrew M. Bishop
+ This file Copyright 2012-2015, 2023 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -797,7 +797,7 @@ static void process_nodes(unsigned char *data,uint32_t length)
  unsigned char *keys_end=NULL,*vals_end=NULL;
  uint32_t keylen=0,vallen=0;
  int64_t lat=0,lon=0;
- TagList *tags=NULL,*result=NULL;
+ TagList *tags=NULL;
 
  while(data<end)
    {
@@ -853,12 +853,7 @@ static void process_nodes(unsigned char *data,uint32_t length)
       }
    }
 
- result=ApplyNodeTaggingRules(tags,id);
-
- ProcessNodeTags(result,id,PBF_LATITUDE(lat),PBF_LONGITUDE(lon),MODE_NORMAL);
-
- DeleteTagList(tags);
- DeleteTagList(result);
+ AddNode(id,PBF_LATITUDE(lat),PBF_LONGITUDE(lon),MODE_NORMAL,tags);
 }
 
 
@@ -878,7 +873,7 @@ static void process_dense_nodes(unsigned char *data,uint32_t length)
  uint32_t idlen=0;
  int64_t id=0;
  int64_t lat=0,lon=0;
- TagList *tags=NULL,*result;
+ TagList *tags=NULL;
 
  while(data<end)
    {
@@ -946,12 +941,7 @@ static void process_dense_nodes(unsigned char *data,uint32_t length)
          }
       }
 
-    result=ApplyNodeTaggingRules(tags,id);
-
-    ProcessNodeTags(result,id,PBF_LATITUDE(lat),PBF_LONGITUDE(lon),MODE_NORMAL);
-
-    DeleteTagList(tags);
-    DeleteTagList(result);
+    AddNode(id,PBF_LATITUDE(lat),PBF_LONGITUDE(lon),MODE_NORMAL,tags);
    }
 }
 
@@ -972,7 +962,7 @@ static void process_ways(unsigned char *data,uint32_t length)
  unsigned char *keys_end=NULL,*vals_end=NULL,*refs_end=NULL;
  uint32_t keylen=0,vallen=0,reflen=0;
  int64_t ref=0;
- TagList *tags=NULL,*result;
+ TagList *tags=NULL;
 
  while(data<end)
    {
@@ -1042,12 +1032,7 @@ static void process_ways(unsigned char *data,uint32_t length)
        AddWayRefs(ref);
       }
 
- result=ApplyWayTaggingRules(tags,id);
-
- ProcessWayTags(result,id,MODE_NORMAL);
-
- DeleteTagList(tags);
- DeleteTagList(result);
+ AddWay(id,MODE_NORMAL,tags);
 }
 
 
@@ -1067,7 +1052,7 @@ static void process_relations(unsigned char *data,uint32_t length)
  unsigned char *keys_end=NULL,*vals_end=NULL,*memids_end=NULL,*types_end=NULL;
  uint32_t keylen=0,vallen=0,rolelen=0,memidlen=0,typelen=0;
  int64_t memid=0;
- TagList *tags=NULL,*result;
+ TagList *tags=NULL;
 
  while(data<end)
    {
@@ -1154,12 +1139,7 @@ static void process_relations(unsigned char *data,uint32_t length)
           AddRelationRefs(0,0,memid,(char*)role);
       }
 
- result=ApplyRelationTaggingRules(tags,id);
-
- ProcessRelationTags(result,id,MODE_NORMAL);
-
- DeleteTagList(tags);
- DeleteTagList(result);
+ AddRelation(id,MODE_NORMAL,tags);
 }
 
 
