@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2015 Andrew M. Bishop
+ This file Copyright 2008-2015, 2023 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -439,12 +439,9 @@ static int nodeType_function(const char *_tag_,int _type_,const char *id,const c
 
  if(_type_&XMLPARSE_TAG_END)
    {
-    TagList *result=ApplyNodeTaggingRules(current_tags,llid);
+    AddNode(llid,latitude,longitude,current_mode,current_tags);
 
-    ProcessNodeTags(result,llid,latitude,longitude,current_mode);
-
-    DeleteTagList(current_tags); current_tags=NULL;
-    DeleteTagList(result);
+    current_tags=NULL;
    }
 
  return(0);
@@ -485,12 +482,9 @@ static int wayType_function(const char *_tag_,int _type_,const char *id)
 
  if(_type_&XMLPARSE_TAG_END)
    {
-    TagList *result=ApplyWayTaggingRules(current_tags,llid);
+    AddWay(llid,current_mode,current_tags);
 
-    ProcessWayTags(result,llid,current_mode);
-
-    DeleteTagList(current_tags); current_tags=NULL;
-    DeleteTagList(result);
+    current_tags=NULL;
    }
 
  return(0);
@@ -531,12 +525,9 @@ static int relationType_function(const char *_tag_,int _type_,const char *id)
 
  if(_type_&XMLPARSE_TAG_END)
    {
-    TagList *result=ApplyRelationTaggingRules(current_tags,llid);
+    AddRelation(llid,current_mode,current_tags);
 
-    ProcessRelationTags(result,llid,current_mode);
-
-    DeleteTagList(current_tags); current_tags=NULL;
-    DeleteTagList(result);
+    current_tags=NULL;
    }
 
  return(0);
