@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2017, 2019 Andrew M. Bishop
+ This file Copyright 2008-2017, 2019, 2024 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -337,13 +337,13 @@ Routino_Output *PrintRoute(Results **results,int nresults,Nodes *nodes,Segments
  /* Create the head of the linked list */
 
  if(option_list_html)
-    listhead=htmllist=calloc(sizeof(Routino_Output),1);
+    listhead=htmllist=calloc(1,sizeof(Routino_Output));
  if(option_list_html_all)
-    listhead=htmlalllist=htmllist=calloc(sizeof(Routino_Output),1);
+    listhead=htmlalllist=htmllist=calloc(1,sizeof(Routino_Output));
  if(option_list_text)
-    listhead=textlist=calloc(sizeof(Routino_Output),1);
+    listhead=textlist=calloc(1,sizeof(Routino_Output));
  if(option_list_text_all)
-    listhead=textalllist=calloc(sizeof(Routino_Output),1);
+    listhead=textalllist=calloc(1,sizeof(Routino_Output));
 
  /* Loop through all the sections of the route and print them */
 
@@ -781,7 +781,7 @@ Routino_Output *PrintRoute(Results **results,int nresults,Nodes *nodes,Segments
                 if(htmlalllist)
                    htmllist=htmlalllist;
 
-                htmllist->next=calloc(sizeof(Routino_Output),1);
+                htmllist->next=calloc(1,sizeof(Routino_Output));
                 htmllist=htmllist->next;
 
                 if(htmlalllist)
@@ -965,7 +965,7 @@ Routino_Output *PrintRoute(Results **results,int nresults,Nodes *nodes,Segments
 
              if(point_count==0) /* first point */
                {
-                textlist->next=calloc(sizeof(Routino_Output),1);
+                textlist->next=calloc(1,sizeof(Routino_Output));
 
                 textlist->bearing=next_bearing_int;
                 textlist->name=strcpy(malloc(strlen(next_waynameraw)+1),next_waynameraw);
@@ -979,7 +979,7 @@ Routino_Output *PrintRoute(Results **results,int nresults,Nodes *nodes,Segments
                }
              else               /* middle point */
                {
-                textlist->next=calloc(sizeof(Routino_Output),1);
+                textlist->next=calloc(1,sizeof(Routino_Output));
 
                 textlist->dist=distance_to_km(cum_distance);
                 textlist->time=duration_to_minutes(cum_duration);
@@ -1023,7 +1023,7 @@ Routino_Output *PrintRoute(Results **results,int nresults,Nodes *nodes,Segments
          {
           if(htmlalllist)
             {
-             htmlalllist->next=calloc(sizeof(Routino_Output),1);
+             htmlalllist->next=calloc(1,sizeof(Routino_Output));
              htmlalllist=htmlalllist->next;
 
              htmlalllist->lon=longitude;
@@ -1069,23 +1069,41 @@ Routino_Output *PrintRoute(Results **results,int nresults,Nodes *nodes,Segments
 
              if(point_count==0) /* first point */
                {
-                fprintf(textallfile,"%10.6f\t%11.6f\t%8d%c\t%s\t%5.3f\t%5.2f\t%5.2f\t%5.1f\t\t\t\n",
-                                    radians_to_degrees(latitude),radians_to_degrees(longitude),
-                                    IsFakeNode(result->node)?(NODE_FAKE-result->node):result->node,
-                                    (resultnodep && IsSuperNode(resultnodep))?'*':' ',type,
-                                    0.0,0.0,0.0,0.0);
+                if(IsFakeNode(result->node))
+                   fprintf(textallfile,"%10.6f\t%11.6f\t%8"PRId32"%c\t%s\t%5.3f\t%5.2f\t%5.2f\t%5.1f\t\t\t\n",
+                                       radians_to_degrees(latitude),radians_to_degrees(longitude),
+                                       (int32_t)((NODE_FAKE-result->node)),
+                                       ' ',type,
+                                       0.0,0.0,0.0,0.0);
+                else
+                   fprintf(textallfile,"%10.6f\t%11.6f\t%8"Pindex_t"%c\t%s\t%5.3f\t%5.2f\t%5.2f\t%5.1f\t\t\t\n",
+                                       radians_to_degrees(latitude),radians_to_degrees(longitude),
+                                       result->node,
+                                       (resultnodep && IsSuperNode(resultnodep))?'*':' ',type,
+                                       0.0,0.0,0.0,0.0);
                }
              else               /* not the first point */
                {
-                fprintf(textallfile,"%10.6f\t%11.6f\t%8d%c\t%s\t%5.3f\t%5.2f\t%5.2f\t%5.1f\t%3d\t%4d\t%s\n",
-                                    radians_to_degrees(latitude),radians_to_degrees(longitude),
-                                    IsFakeNode(result->node)?(NODE_FAKE-result->node):result->node,
-                                    (resultnodep && IsSuperNode(resultnodep))?'*':' ',type,
-                                    distance_to_km(seg_distance),duration_to_minutes(seg_duration),
-                                    distance_to_km(cum_distance),duration_to_minutes(cum_duration),
-                                    speed_to_kph(seg_speed),
-                                    bearing_int,
-                                    waynameraw);
+                if(IsFakeNode(result->node))
+                   fprintf(textallfile,"%10.6f\t%11.6f\t%8"PRId32"%c\t%s\t%5.3f\t%5.2f\t%5.2f\t%5.1f\t%3d\t%4d\t%s\n",
+                                       radians_to_degrees(latitude),radians_to_degrees(longitude),
+                                       (int32_t)(NODE_FAKE-result->node),
+                                       ' ',type,
+                                       distance_to_km(seg_distance),duration_to_minutes(seg_duration),
+                                       distance_to_km(cum_distance),duration_to_minutes(cum_duration),
+                                       speed_to_kph(seg_speed),
+                                       bearing_int,
+                                       waynameraw);
+                else
+                   fprintf(textallfile,"%10.6f\t%11.6f\t%8"Pindex_t"%c\t%s\t%5.3f\t%5.2f\t%5.2f\t%5.1f\t%3d\t%4d\t%s\n",
+                                       radians_to_degrees(latitude),radians_to_degrees(longitude),
+                                       result->node,
+                                       (resultnodep && IsSuperNode(resultnodep))?'*':' ',type,
+                                       distance_to_km(seg_distance),duration_to_minutes(seg_duration),
+                                       distance_to_km(cum_distance),duration_to_minutes(cum_duration),
+                                       speed_to_kph(seg_speed),
+                                       bearing_int,
+                                       waynameraw);
                }
 
              if(important==ROUTINO_POINT_WAYPOINT)
@@ -1098,7 +1116,7 @@ Routino_Output *PrintRoute(Results **results,int nresults,Nodes *nodes,Segments
                 ;
              else               /* not the first point */
                {
-                textalllist->next=calloc(sizeof(Routino_Output),1);
+                textalllist->next=calloc(1,sizeof(Routino_Output));
                 textalllist=textalllist->next;
 
                 textalllist->dist=distance_to_km(cum_distance);
