@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2010-2015, 2018, 2019, 2020, 2022 Andrew M. Bishop
+ This file Copyright 2010-2015, 2018, 2019, 2020, 2022, 2024, 2025 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -75,6 +75,8 @@ RelationsX *NewRelationList(int append,int readonly)
 {
  RelationsX *relationsx;
 
+ logassert(sizeof(relation_t)>=sizeof(index_t),"Size of relation_t type must be at least as large as size of index_t type.");
+
  relationsx=(RelationsX*)calloc_logassert(1,sizeof(RelationsX));
 
 
@@ -650,8 +652,7 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx,int keep)
 
  do
    {
-    int ways=0,relations=0;
-    index_t i;
+    index_t i,ways=0,relations=0;
 
     /* Print the start message */
 
@@ -1190,7 +1191,7 @@ void RemovePrunedTurnRelations(RelationsX *relationsx,NodesX *nodesx)
     relationx.via =nodesx->pdata[relationx.via];
     relationx.to  =nodesx->pdata[relationx.to];
 
-    if(relationx.from==NO_NODE || relationx.via==NO_NODE || relationx.to==NO_NODE)
+    if(relationx.from==NO_WAY || relationx.via==NO_NODE || relationx.to==NO_WAY)
        pruned++;
     else
       {
