@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2016, 2018 Andrew M. Bishop
+ This file Copyright 2008-2016, 2018, 2024 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -339,7 +339,7 @@ int main(int argc,char** argv)
     exit(EXIT_FAILURE);
    }
 
- waypoints=calloc(sizeof(Routino_Waypoint*),nwaypoints+2);
+ waypoints=calloc(nwaypoints+2,sizeof(Routino_Waypoint*));
 
  /* Load in the routing database */
 
