@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2015-2017, 2019, 2020 Andrew M. Bishop
+ This file Copyright 2015-2017, 2019, 2020, 2024 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -140,7 +140,7 @@ DLL_PUBLIC Routino_Database *Routino_LoadDatabase(const char *dirname,const char
    }
  else
    {
-    database=calloc(sizeof(Routino_Database),1);
+    database=calloc(1,sizeof(Routino_Database));
 
     database->nodes    =LoadNodeList    (nodes_filename);
     database->segments =LoadSegmentList (segments_filename);
@@ -543,7 +543,7 @@ DLL_PUBLIC Routino_Waypoint *Routino_FindWaypoint(Routino_Database *database,Rou
     return(NULL);
    }
 
- waypoint=calloc(sizeof(Routino_Waypoint),1);
+ waypoint=calloc(1,sizeof(Routino_Waypoint));
 
  waypoint->segment=FindClosestSegment(database->nodes,database->segments,database->ways,
                                       degrees_to_radians(latitude),degrees_to_radians(longitude),distmax,profile,
@@ -676,7 +676,7 @@ DLL_PUBLIC Routino_Output *Routino_CalculateRoute(Routino_Database *database,Rou
 
  /* Loop through all pairs of waypoints */
 
- results=calloc(sizeof(Results*),nwaypoints);
+ results=calloc(nwaypoints,sizeof(Results*));
 
  for(this_waypoint=first_waypoint;this_waypoint!=(last_waypoint+inc_dec_waypoint);this_waypoint+=inc_dec_waypoint)
    {
