@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2015 Andrew M. Bishop
+ This file Copyright 2008-2015, 2022 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -69,7 +69,7 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
  if(!nodesx->super)
    {
     nodesx->super=AllocBitMask(nodesx->number);
-    log_malloc(nodesx->super,LengthBitMask(nodesx->number)*sizeof(BitMask));
+    log_malloc(nodesx->super,SizeBitMask(nodesx->number));
 
     SetAllBits(nodesx->super,nodesx->number);
    }
