@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2010-2015 Andrew M. Bishop
+ This file Copyright 2010-2015, 2024 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -534,7 +534,7 @@ TaggingRuleList *AppendTaggingRule(TaggingRuleList *rules,const char *k,const ch
 
  rules->rules[rules->nrules-1].message=NULL;
 
- rules->rules[rules->nrules-1].rulelist=(TaggingRuleList*)calloc(sizeof(TaggingRuleList),1);
+ rules->rules[rules->nrules-1].rulelist=(TaggingRuleList*)calloc(1,sizeof(TaggingRuleList));
 
  return(rules->rules[rules->nrules-1].rulelist);
 }
@@ -624,7 +624,7 @@ void DeleteTaggingRuleList(TaggingRuleList *rules)
 
 TagList *NewTagList(void)
 {
- return((TagList*)calloc(sizeof(TagList),1));
+ return((TagList*)calloc(1,sizeof(TagList)));
 }
 
 
