@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2012-2015 Andrew M. Bishop
+ This file Copyright 2012-2015, 2025 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -96,7 +96,7 @@ int Uncompress_Bzip2(int filefd)
 
 #else /* USE_BZIP2 */
 
- logassert(0,"No bzip2 compression support available (re-compile and try again)");
+ logassert(0,"No bzip2 compression support available (re-compile and try again).");
 
  return(0);
 
@@ -127,7 +127,7 @@ int Uncompress_Gzip(int filefd)
 
 #else /* USE_GZIP */
 
- logassert(0,"No gzip compression support available (re-compile and try again)");
+ logassert(0,"No gzip compression support available (re-compile and try again).");
 
  return(0);
 
@@ -158,7 +158,7 @@ int Uncompress_Xz(int filefd)
 
 #else /* USE_XZ */
 
- logassert(0,"No xz compression support available (re-compile and try again)");
+ logassert(0,"No xz compression support available (re-compile and try again).");
 
  return(0);
 
@@ -264,7 +264,7 @@ static void uncompress_bzip2_pipe(int filefd,int pipefd)
  int state;
 
  if(BZ2_bzDecompressInit(&bz,0,0)!=BZ_OK)
-    exit(EXIT_FAILURE);
+    exit(EXIT_FAILURE); /* no error message because in sub-process */
 
  do
    {
@@ -287,7 +287,7 @@ static void uncompress_bzip2_pipe(int filefd,int pipefd)
     state=BZ2_bzDecompress(&bz);
 
     if(state!=BZ_OK && state!=BZ_STREAM_END)
-       exit(EXIT_FAILURE);
+       exit(EXIT_FAILURE); /* no error message because in sub-process */
 
     if(bz.avail_out<sizeof(outbuffer))
       {
@@ -302,7 +302,7 @@ static void uncompress_bzip2_pipe(int filefd,int pipefd)
           m=write(pipefd,p,n);
 
           if(m<=0)
-             exit(EXIT_FAILURE);
+             exit(EXIT_FAILURE); /* no error message because in sub-process */
 
           p+=m;
           n-=m;
@@ -312,7 +312,7 @@ static void uncompress_bzip2_pipe(int filefd,int pipefd)
  while(state!=BZ_STREAM_END);
 
  if(BZ2_bzDecompressEnd(&bz)!=BZ_OK)
-    exit(EXIT_FAILURE);
+    exit(EXIT_FAILURE); /* no error message because in sub-process */
 
  exit(EXIT_SUCCESS);
 }
@@ -338,7 +338,7 @@ static void uncompress_gzip_pipe(int filefd,int pipefd)
  int state;
 
  if(inflateInit2(&z,15+32)!=Z_OK)
-    exit(EXIT_FAILURE);
+    exit(EXIT_FAILURE); /* no error message because in sub-process */
 
  do
    {
@@ -361,9 +361,7 @@ static void uncompress_gzip_pipe(int filefd,int pipefd)
     state=inflate(&z,Z_NO_FLUSH);
 
     if(state!=Z_OK && state!=Z_STREAM_END)
-      {
-       exit(EXIT_FAILURE);
-      }
+       exit(EXIT_FAILURE); /* no error message because in sub-process */
 
     if(z.avail_out<sizeof(outbuffer))
       {
@@ -378,7 +376,7 @@ static void uncompress_gzip_pipe(int filefd,int pipefd)
           m=write(pipefd,p,n);
 
           if(m<=0)
-             exit(EXIT_FAILURE);
+             exit(EXIT_FAILURE); /* no error message because in sub-process */
 
           p+=m;
           n-=m;
@@ -388,7 +386,7 @@ static void uncompress_gzip_pipe(int filefd,int pipefd)
  while(state!=Z_STREAM_END);
 
  if(inflateEnd(&z)!=Z_OK)
-    exit(EXIT_FAILURE);
+    exit(EXIT_FAILURE); /* no error message because in sub-process */
 
  exit(EXIT_SUCCESS);
 }
@@ -414,7 +412,7 @@ static void uncompress_xz_pipe(int filefd,int pipefd)
  lzma_ret retval;
 
  if(lzma_stream_decoder(&lzma,UINT64_MAX,0)!=LZMA_OK)
-    exit(EXIT_FAILURE);
+    exit(EXIT_FAILURE); /* no error message because in sub-process */
 
  do
    {
@@ -437,9 +435,7 @@ static void uncompress_xz_pipe(int filefd,int pipefd)
     retval=lzma_code(&lzma,LZMA_RUN);
 
     if(retval!=LZMA_OK && retval!=LZMA_STREAM_END)
-      {
-       exit(EXIT_FAILURE);
-      }
+       exit(EXIT_FAILURE); /* no error message because in sub-process */
 
     if(lzma.avail_out<sizeof(outbuffer))
       {
@@ -454,7 +450,7 @@ static void uncompress_xz_pipe(int filefd,int pipefd)
           m=write(pipefd,p,n);
 
           if(m<=0)
-             exit(EXIT_FAILURE);
+             exit(EXIT_FAILURE); /* no error message because in sub-process */
 
           p+=m;
           n-=m;
@@ -465,7 +461,7 @@ static void uncompress_xz_pipe(int filefd,int pipefd)
 
  lzma_end(&lzma);
 
- exit(EXIT_SUCCESS);
+ exit(EXIT_SUCCESS); /* no error message because in sub-process */
 }
 
 #endif /* USE_XZ */
