@@ -5,7 +5,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2010-2015, 2019 Andrew M. Bishop
+ This file Copyright 2010-2015, 2019, 2025 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -236,7 +236,10 @@ static int elementType_function(const char *_tag_,int _type_,const char *name,co
        tagsx[i]->nsubtagsx++;
 
        if(tagsx[i]->nsubtagsx==XMLPARSE_MAX_SUBTAGS)
-         {fprintf(stderr,"Too many subtags seen for type '%s'.\n",currenttype); exit(1);}
+         {
+          fprintf(stderr,"Error: Too many subtags (%d) seen for type '%s'.\n",XMLPARSE_MAX_SUBTAGS,currenttype);
+          exit(EXIT_FAILURE);
+         }
       }
 
  return(0);
@@ -310,7 +313,10 @@ static int attributeType_function(const char *_tag_,int _type_,const char *name,
        tagsx[i]->nattributes++;
 
        if(tagsx[i]->nattributes==XMLPARSE_MAX_ATTRS)
-         {fprintf(stderr,"Too many attributes seen for type '%s'.\n",currenttype); exit(1);}
+         {
+          fprintf(stderr,"Error: Too many attributes (%d) seen for type '%s'.\n",XMLPARSE_MAX_ATTRS,currenttype);
+          exit(EXIT_FAILURE);
+         }
       }
 
  return(0);
@@ -329,8 +335,8 @@ int main(int argc,char **argv)
 
  if(ParseXML(STDIN_FILENO,xml_toplevel_tags,XMLPARSE_UNKNOWN_ATTR_IGNORE))
    {
-    fprintf(stderr,"Cannot parse XML file - exiting.\n");
-    exit(1);
+    fprintf(stderr,"Error: Cannot parse XML file - exiting.\n");
+    exit(EXIT_FAILURE);
    }
 
  /* Print the header */
