@@ -4,7 +4,7 @@
 #
 # Part of the Routino routing software.
 #
-# This file Copyright 2014 Andrew M. Bishop
+# This file Copyright 2014-2015 Andrew M. Bishop
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -20,16 +20,13 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-# Use the perl encoding/decoding functions
-use Encode qw(decode encode);
-
 use strict;
 
 # Constants
 
 my @translation_files=(<translation.*.txt>);
-my $xml_file="../data/translations.xml";
-my $output_dir="../www/routino";
+my $xml_output_file="../../xml/routino-translations.xml";
+my $html_output_dir="../www/routino";
 
 my @html_template_files=(<*.html>);
 
@@ -37,7 +34,11 @@ my %languages=();
 my %translations=();
 
 
-# Read in the translations (HTML)
+# Sort so that English is first
+
+@translation_files=sort {if($a eq "translation.en.txt"){return -1;} if($b eq "translation.en.txt"){return 1;} return $a <=> $b;} @translation_files;
+
+# Read in the translations
 
 foreach my $translation_file (@translation_files)
   {
@@ -68,7 +69,7 @@ foreach my $translation_file (@translation_files)
       next if(m%^#%);
       next if(m%^$%);
 
-      # Single line entries
+      # Single line HTML entries
 
       if(m%\@\@%)
         {
@@ -82,16 +83,18 @@ foreach my $translation_file (@translation_files)
            {
             $translations{$language}->{html}++;
             $translations{$language}->{codes}->{$code}={};
-            $translations{$language}->{codes}->{$code}->{text}=encode('ascii',decode('UTF-8',$text),Encode::FB_HTMLCREF);
-            $translations{$language}->{codes}->{$code}->{used}=0;
+            $translations{$language}->{codes}->{$code}->{text}=$text;
+            $translations{$language}->{codes}->{$code}->{usedX}=0;
+            $translations{$language}->{codes}->{$code}->{usedR}=0;
+            $translations{$language}->{codes}->{$code}->{usedV}=0;
            }
         }
 
-      # Multi-line entries
+      # Multi-line HTML entries
 
-      if(m%\$\$([^\$]+)\$\$%)
+      if(m%(\$\$[^\$]+\$\$)%)
         {
-         my($code,$text)=("@@".$1."@@","");
+         my($code,$text)=($1,"");
 
          while(<FILE>)
            {
@@ -110,8 +113,39 @@ foreach my $translation_file (@translation_files)
            {
             $translations{$language}->{html}++;
             $translations{$language}->{codes}->{$code}={};
-            $translations{$language}->{codes}->{$code}->{text}=encode('ascii',decode('UTF-8',$text),Encode::FB_HTMLCREF);
-            $translations{$language}->{codes}->{$code}->{used}=0;
+            $translations{$language}->{codes}->{$code}->{text}=$text;
+            $translations{$language}->{codes}->{$code}->{usedX}=0;
+            $translations{$language}->{codes}->{$code}->{usedR}=0;
+            $translations{$language}->{codes}->{$code}->{usedV}=0;
+           }
+        }
+
+      # Single line XML entries
+
+      if(m%\%\%%)
+        {
+         my($code,$text)=split("\t");
+
+         if(defined $translations{$language}->{codes}->{$code})
+           {
+            print STDERR "Language: $language DUPLICATED codeword '$code'\n";
+           }
+         else
+           {
+            $translations{$language}->{xml}++;
+            $translations{$language}->{codes}->{$code}={};
+            $translations{$language}->{codes}->{$code}->{text}=$text;
+            $translations{$language}->{codes}->{$code}->{usedX}=0;
+            $translations{$language}->{codes}->{$code}->{usedR}=0;
+            $translations{$language}->{codes}->{$code}->{usedV}=0;
+           }
+
+         my($n_strings_en)=$translations{en}->{codes}->{$code}->{text} =~ s/%s/%s/g;
+         my($n_strings)   =$text =~ s/%s/%s/g;
+
+         if($n_strings != $n_strings_en)
+           {
+            print STDERR "Language: $language WRONG number of '%s' in text '$text' ($translations{en}->{codes}->{$code}->{text})\n";
            }
         }
      }
@@ -120,48 +154,6 @@ foreach my $translation_file (@translation_files)
   }
 
 
-# Read in the translations (XML)
-
-open(XML,"<$xml_file");
-
-my $language="";
-
-while(<XML>)
-  {
-   if(m%<language +lang=["']([^"']+)["'] *>%)
-     {
-      # Add to list of languages
-
-      $language=$1;
-
-      if(! defined $languages{$language})
-        {
-         $languages{$language}=1;
-
-         $translations{$language}={};
-         $translations{$language}->{codes}={};
-         $translations{$language}->{html}=0;
-         $translations{$language}->{xml}=0;
-        }
-
-      next;
-     }
-
-   $language="" if(m%</language *>%);
-
-   next if(!$language);
-
-   # Count the phrases
-
-   if(m%<[a-z]+% && m% +(text|string)=["']([^"']+)["']%)
-     {
-      $translations{$language}->{xml}++;
-     }
-  }
-
-close(XML);
-
-
 # Sort out the languages
 
 my @languages=();
@@ -174,10 +166,15 @@ foreach my $language (sort (keys %languages))
   }
 
 
-# Create the files
+# Create the HTML files
 
 foreach my $html_template_file (@html_template_files)
   {
+   my $usedtype="";
+
+   $usedtype="R" if($html_template_file =~ m%router%);
+   $usedtype="V" if($html_template_file =~ m%visualiser%);
+
    foreach my $language (@languages)
      {
       next if(!$translations{$language}->{html});
@@ -188,7 +185,7 @@ foreach my $html_template_file (@html_template_files)
       my $language_meta_string="";
 
       open(HTML_IN ,"<$html_template_file");
-      open(HTML_OUT,">$output_dir/$html_template_file.$language");
+      open(HTML_OUT,">$html_output_dir/$html_template_file.$language");
 
       while(<HTML_IN>)
         {
@@ -196,7 +193,7 @@ foreach my $html_template_file (@html_template_files)
 
          # Language selection - special handling
 
-         if($line =~ m%\$\$LANGUAGES-META\$\$%)
+         if($line =~ m%\*\*LANGUAGES-META\*\*%)
            {
             $language_meta=1-$language_meta;
 
@@ -211,15 +208,15 @@ foreach my $html_template_file (@html_template_files)
 
                   if($language eq $language2)
                     {
-                     $line =~ s%CHECKED-XX%checked%g;
+                     $line =~ s%~~CHECKED~~%checked%g;
                     }
                   else
                     {
-                     $line =~ s%CHECKED-XX%%g;
+                     $line =~ s%~~CHECKED~~%%g;
                     }
 
-                  $line =~ s%xx%$language2%g;
-                  $line =~ s%XX%$LANGUAGE2%g;
+                  $line =~ s%~~lang~~%$language2%g;
+                  $line =~ s%~~LANG~~%$LANGUAGE2%g;
 
                   if(!$translations{$language2}->{html})
                     {
@@ -231,10 +228,15 @@ foreach my $html_template_file (@html_template_files)
                      $line =~ s%<input .+>%%;
                     }
 
-                  foreach my $code (keys $translations{$language}->{codes})
+                  foreach my $code (keys %{$translations{$language2}->{codes}})
                     {
                      if($line =~ s%$code%$translations{$language2}->{codes}->{$code}->{text}%g)
-                       {$translations{$language2}->{codes}->{$code}->{used} = 1;}
+                       {$translations{$language2}->{codes}->{$code}->{"used$usedtype"} = 1;}
+                    }
+
+                  if($line =~ m%((\@\@|\$\$|\*\*|\~\~)[^\@\$*~]+(\@\@|\$\$|\*\*|\~\~))%)
+                    {
+                     print STDERR "   Unmatched codeword '$1' in line: $line";
                     }
 
                   # Remove un-needed spaces
@@ -258,22 +260,22 @@ foreach my $html_template_file (@html_template_files)
 
          # Replace with translated phrases
 
-         foreach my $code (keys $translations{$language}->{codes})
+         foreach my $code (keys %{$translations{$language}->{codes}})
            {
-            if($line =~ s%$code%$translations{$language}->{codes}->{$code}->{text}%g)
-              {$translations{$language}->{codes}->{$code}->{used} = 1;}
+            if($line =~ s%\Q$code\E%$translations{$language}->{codes}->{$code}->{text}%g)
+              {$translations{$language}->{codes}->{$code}->{"used$usedtype"} = 1;}
            }
 
          # Replace what is left with English phrases
 
-         foreach my $code (keys $translations{$languages[0]}->{codes})
+         foreach my $code (keys %{$translations{$languages[0]}->{codes}})
            {
-            $line =~ s%$code%$translations{$languages[0]}->{codes}->{$code}->{text}%g;
+            $line =~ s%\Q$code\E%$translations{$languages[0]}->{codes}->{$code}->{text}%g;
            }
 
-         if($line =~ m%\@\@%)
+         if($line =~ m%((\@\@|\$\$|\*\*|\~\~)[^\@\$*~]+(\@\@|\$\$|\*\*|\~\~))%)
            {
-            print STDERR "   Unmatched codeword in line: $line";
+            print STDERR "   Unmatched codeword '$1' in line: $line";
            }
 
          # Remove un-needed spaces
@@ -291,13 +293,99 @@ foreach my $html_template_file (@html_template_files)
   }
 
 
-# Check the languages
+# Create the XML file
+
+open(XML_OUT,">$xml_output_file");
+
+open(XML_IN ,"<translations-head.xml");
+
+while(<XML_IN>)
+  {
+   print XML_OUT;
+  }
+
+close(XML_IN);
 
 foreach my $language (@languages)
   {
-   foreach my $code (keys $translations{$language}->{codes})
+   next if(!$translations{$language}->{xml});
+
+   print "Language: $language File: translations.xml\n";
+
+   open(XML_IN ,"<translations-body.xml");
+
+   while(<XML_IN>)
      {
-      if(! $translations{$language}->{codes}->{$code}->{used})
+      my $line=$_;
+
+      $line =~ s%~~lang~~%$language%g;
+      $line =~ s%~~language~~%$translations{$language}->{codes}->{'@@LANGUAGE@@'}->{text}%g;
+
+      # Replace with translated phrases
+
+      foreach my $code (keys %{$translations{$language}->{codes}})
+        {
+         if($line =~ s%$code%$translations{$language}->{codes}->{$code}->{text}%g)
+           {$translations{$language}->{codes}->{$code}->{usedX} = 1;}
+        }
+
+      # Replace what is left with a note about missing translations
+
+      if($line =~ m%\%\%%)
+        {
+         foreach my $code (keys %{$translations{$languages[0]}->{codes}})
+           {
+            $line =~ s%$code%$translations{$languages[0]}->{codes}->{$code}->{text}%g;
+           }
+
+         $line =~ s%<%<!-- TRANSLATION REQUIRED: %;
+         $line =~ s%>% -->%;
+
+         if($line =~ m%((\%\%|\~\~)[^\%~]+(\%\%|\~\~))%)
+           {
+            print STDERR "   Unmatched codeword '$1' in line: $line";
+           }
+        }
+
+      print XML_OUT $line;
+     }
+
+   close(XML_IN);
+  }
+
+open(XML_IN ,"<translations-tail.xml");
+
+while(<XML_IN>)
+  {
+   print XML_OUT;
+  }
+
+close(XML_IN);
+
+close(XML_OUT);
+
+
+# Check the languages and usage
+
+my %usedX=();
+my %usedR=();
+my %usedV=();
+
+foreach my $language (@languages)
+  {
+   $usedX{$language}=0;
+   $usedR{$language}=0;
+   $usedV{$language}=0;
+
+   foreach my $code (keys %{$translations{$language}->{codes}})
+     {
+      $usedX{$language}+=$translations{$language}->{codes}->{$code}->{usedX};
+      $usedR{$language}+=$translations{$language}->{codes}->{$code}->{usedR};
+      $usedV{$language}+=$translations{$language}->{codes}->{$code}->{usedV};
+
+      if(! $translations{$language}->{codes}->{$code}->{usedX} &&
+         ! $translations{$language}->{codes}->{$code}->{usedR} &&
+         ! $translations{$language}->{codes}->{$code}->{usedV})
         {
          print STDERR "Language: $language UNUSED codeword: $code\n";
         }
@@ -313,15 +401,17 @@ print "Translation Coverage\n";
 print "====================\n";
 
 print "\n";
-print "            Number      Fraction\n";
-print "Language  HTML   XML   HTML   XML\n";
-print "--------  ----   ---   ----   ---\n";
+print "           Number      Percentage Complete\n";
+print "Language  XML HTML    XML router visualiser\n";
+print "--------  --- ----    --- ------ ----------\n";
 
 foreach my $language (@languages)
   {
-   printf("%-6s     %3d   %3d  %4.0f%% %4.0f%%\n",$language,
-                                                  $translations{$language}->{html},
-                                                  $translations{$language}->{xml},
-                                                  100.0*$translations{$language}->{html}/$translations{$languages[0]}->{html},
-                                                  100.0*$translations{$language}->{xml} /$translations{$languages[0]}->{xml});
+   printf("%-6s    %3d  %3d  %4.0f%%  %4.0f%%  %4.0f%%\n",
+          $language,
+          $translations{$language}->{xml},
+          $translations{$language}->{html},
+          100.0*$usedX{$language}/$usedX{$languages[0]},
+          100.0*$usedR{$language}/$usedR{$languages[0]},
+          100.0*$usedV{$language}/$usedV{$languages[0]})
   }
