@@ -124,6 +124,10 @@ Select start and end points (click on the marker icons below), select routing pr
 <td><a id="lang_ru_url" href="router.html.ru" title="Страница на русском языке">Русский</a>
 <td>(RU)
 <td><input name="language" type="radio" value="ru" onchange="formSetLanguage();" >
+<tr>
+<td><a id="lang_sk_url" href="router.html.sk" title="Stránka v slovenčine">Slovenčina</a>
+<td>(SK)
+<td><input name="language" type="radio" value="sk" onchange="formSetLanguage();" >
 </table>
 <a target="translation" href="http://www.routino.org/translations/">Routino Translations</a>
 </div>
