@@ -3,7 +3,7 @@
 //
 // Part of the Routino routing software.
 //
-// This file Copyright 2008-2019 Andrew M. Bishop
+// This file Copyright 2008-2020 Andrew M. Bishop
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Affero General Public License as published by
@@ -67,7 +67,7 @@ for(var marker=1;marker<=mapprops.maxmarkers;marker++)
 
 var legal={"^lon"             : "^[-0-9.]+$",
            "^lat"             : "^[-0-9.]+$",
-           "^zoom"            : "^[0-9]+$",
+           "^zoom"            : "^[-0-9.]+$",
 
            "^lon[1-9]"        : "^[-0-9.]+$",
            "^lat[1-9]"        : "^[-0-9.]+$",
@@ -165,6 +165,12 @@ function form_init()            // called from router.html
  // Fill in the waypoints
 
  vismarkers=0;
+ urlmarkers=0;
+
+ minlat=90;
+ maxlat=-90;
+ minlon=180;
+ maxlon=-180;
 
  for(var marker=mapprops.maxmarkers;marker>=1;marker--)
    {
@@ -179,11 +185,21 @@ function form_init()            // called from router.html
        formSetSearch(marker,search);
        formSetCoords(marker,lon,lat);
 
+       lat=Number(lat);
+       lon=Number(lon);
+
+       if(lat<minlat) minlat=lat;
+       if(lat>maxlat) maxlat=lat;
+
+       if(lon<minlon) minlon=lon;
+       if(lon>maxlon) maxlon=lon;
+
        markerAddMap(marker);
 
        markerSearch(marker);
 
        vismarkers++;
+       urlmarkers++;
       }
     else if(lon !== undefined && lat !== undefined && lon !== "" && lat !== "")
       {
@@ -191,11 +207,21 @@ function form_init()            // called from router.html
 
        formSetCoords(marker,lon,lat);
 
+       lat=Number(lat);
+       lon=Number(lon);
+
+       if(lat<minlat) minlat=lat;
+       if(lat>maxlat) maxlat=lat;
+
+       if(lon<minlon) minlon=lon;
+       if(lon>maxlon) maxlon=lon;
+
        markerAddMap(marker);
 
        markerCoords(marker);
 
        vismarkers++;
+       urlmarkers++;
       }
     else if(search !== undefined && search !== "")
       {
@@ -234,6 +260,25 @@ function form_init()            // called from router.html
  else
     formSetLoopReverse("reverse",false);
 
+ // Zoom the map
+
+ if(urlmarkers>1)
+   {
+    var lon =args["lon"];
+    var lat =args["lat"];
+    var zoom=args["zoom"];
+
+    if(lon === undefined || lat === undefined || zoom === undefined)
+      {
+       var deltalon = 0.025 * ( maxlon - minlon );
+       var deltalat = 0.025 * ( maxlat - minlat );
+
+       var markerextent=L.latLngBounds(L.latLng(minlat-deltalat,minlon-deltalon),L.latLng(maxlat+deltalat,maxlon+deltalon));
+
+       map.fitBounds(markerextent);
+      }
+   }
+
  // Update the transport type with the URL settings which updates all HTML forms to defaults.
 
  var transport=routino.transport;
@@ -281,7 +326,7 @@ function form_init()            // called from router.html
  var cookies=document.cookie.split("; ");
 
  for(var cookie=0;cookie<cookies.length;cookie++)
-    if(cookies[cookie].substr(0,"Routino-home".length)=="Routino-home")
+    if(cookies[cookie].startsWith("Routino-home"))
       {
        var data=cookies[cookie].split(/[=:;]/);
 
@@ -300,7 +345,7 @@ function form_init()            // called from router.html
        markerMoveHome(1);
    }
 
- updateURLs();
+ updateURLs(true);
 
  updateSearchButtons();
 }
@@ -344,7 +389,7 @@ function formSetLanguage(value) // called from router.html (with no arguments)
     routino.language=value;
    }
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -378,7 +423,7 @@ function formSetTransport(value) // called from router.html
 
  paramschanged=true;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -411,7 +456,7 @@ function formSetHighway(type,value) // called from router.html (with one argumen
 
  paramschanged=true;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -448,7 +493,7 @@ function formSetSpeed(type,value) // called from router.html (with one argument)
 
  paramschanged=true;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -483,7 +528,7 @@ function formSetProperty(type,value) // called from router.html (with one argume
 
  paramschanged=true;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -538,7 +583,7 @@ function formSetRestriction(type,value) // called from router.html (with one arg
 
  paramschanged=true;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -564,7 +609,7 @@ function formSetCoords(marker,lon,lat) // called from router.html (with one argu
     routino.point[marker].lon="";
     routino.point[marker].lat="";
 
-    updateURLs();
+    updateURLs(true);
    }
  else
    {
@@ -576,6 +621,8 @@ function formSetCoords(marker,lon,lat) // called from router.html (with one argu
 
        lon=lonlat.lon;
       }
+    else
+       lon=Number(lon);
 
     if(lon<-180) lon=-180;
     if(lon>+180) lon=+180;
@@ -586,6 +633,8 @@ function formSetCoords(marker,lon,lat) // called from router.html (with one argu
 
        lat=lonlat.lat;
       }
+    else
+       lat=Number(lat);
 
     if(lat<-90 ) lat=-90 ;
     if(lat>+90 ) lat=+90 ;
@@ -647,7 +696,7 @@ function formSetLoopReverse(type,value) // called from router.html (with one arg
  else
     routino.reverse=value;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -739,11 +788,15 @@ function buildMapArguments()
 // Update the URLs
 //
 
-function updateURLs()
+function updateURLs(addhistory)
 {
- var urlargs1=buildURLArguments(true);
- var urlargs2=buildURLArguments(false);
- var mapargs=buildMapArguments();
+ var urlargs =buildURLArguments(false);
+ var mapargs =buildMapArguments();
+ var langargs="language=" + routino.language;
+ var libargs =";library=" + mapprops.library;
+
+ if(!mapprops.libraries)
+    libargs="";
 
  var links=document.getElementsByTagName("a");
 
@@ -752,17 +805,23 @@ function updateURLs()
     var element=links[i];
 
     if(element.id == "permalink_url")
-       element.href=location.pathname + "?" + urlargs1 + ";" + mapargs;
+       element.href=location.pathname + "?" + urlargs + ";" + langargs + ";" + mapargs + libargs;
 
     if(element.id == "visualiser_url")
-       element.href="visualiser.html" + "?" + mapargs;
+       if(location.pathname.match(/router\.html\.([a-zA-Z-]+)$/))
+          element.href="visualiser.html." + RegExp.$1 + "?" + mapargs + libargs;
+       else
+          element.href="visualiser.html" + "?" + mapargs + libargs;
 
     if(element.id == "edit_url")
        element.href=mapprops.editurl + "?" + mapargs;
 
     if(element.id.match(/^lang_([a-zA-Z-]+)_url$/))
-       element.href="router.html" + "." + RegExp.$1 + "?" + urlargs2 + ";" + mapargs;
+       element.href="router.html" + "." + RegExp.$1 + "?" + urlargs + ";" + mapargs + libargs;
    }
+
+ if(addhistory)
+    history.replaceState(null, null, location.pathname + "?" + urlargs + ";" + langargs + libargs);
 }
 
 
@@ -889,7 +948,7 @@ function map_init()             // called from router.html
 
  // Move the map
 
- map.on("moveend", updateURLs);
+ map.on("moveend", (function() { updateURLs(false);}));
 
  var lon =args["lon"];
  var lat =args["lat"];
@@ -897,6 +956,10 @@ function map_init()             // called from router.html
 
  if(lon !== undefined && lat !== undefined && zoom !== undefined)
    {
+    lat  = Number(lat);
+    lon  = Number(lon);
+    zoom = Number.parseInt(Number(zoom)+0.5);
+
     if(lon<mapprops.westedge) lon=mapprops.westedge;
     if(lon>mapprops.eastedge) lon=mapprops.eastedge;
 
@@ -921,7 +984,7 @@ function map_init()             // called from router.html
     edit_url.href=mapprops.editurl;
    }
 
- updateURLs();
+ updateURLs(false);
 }
 
 
@@ -936,14 +999,14 @@ function dragMarkerMove(marker,event)
 
 
 //
-// Callback for completing a drag occuring on the map.
+// Callback for completing a marker drag on the map.
 //
 
 function dragMarkerComplete(marker,event)
 {
  dragMarkerSetForm(marker);
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -965,7 +1028,7 @@ function dragMarkerSetForm(marker)
 
 var dragged_waypoint=null,dragged_marker=null;
 var dragged_waypoint_over=null,dragged_marker_over=null;
- var dragged_icon_x,dragged_icon_y;
+var dragged_icon_x,dragged_icon_y;
 
 //
 // Drag a waypoint up or down the list.
@@ -1180,7 +1243,7 @@ function markerAddMap(marker)
 
  markersmoved=true;
 
- updateURLs();
+ updateURLs(true);
 
  updateSearchButtons();
 }
@@ -1201,7 +1264,7 @@ function markerRemoveMap(marker)
 
  markersmoved=true;
 
- updateURLs();
+ updateURLs(true);
 
  updateSearchButtons();
 }
@@ -1261,8 +1324,8 @@ function markerRecentre(marker) // called from router.html
 
  clearSearchResult(marker);
 
- var lon=routino.point[marker].lon;
- var lat=routino.point[marker].lat;
+ var lon=Number(routino.point[marker].lon);
+ var lat=Number(routino.point[marker].lat);
 
  var lonlat = L.latLng(lat,lon);
 
@@ -1475,14 +1538,14 @@ function markerSetClearHome(marker,home)
     homelat=null;
     homelon=null;
 
-    cookie="Routino-home=unset";
+    cookie="Routino-home=";
 
     date.setUTCFullYear(date.getUTCFullYear()-1);
 
     routino.point[marker].home=false;
    }
 
- document.cookie=cookie + ";expires=" + date.toGMTString();
+ document.cookie=cookie + ";Expires=" + date.toGMTString() + ";SameSite=Strict";
 
  updateIcon(marker);
 
@@ -1606,7 +1669,7 @@ function markersReverse()       // called from router.html
 
  markersmoved=true;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -1637,7 +1700,7 @@ function markersLoop()          // called from router.html
 
  markersmoved=true;
 
- updateURLs();
+ updateURLs(true);
 
  updateSearchButtons();
 }
@@ -1696,6 +1759,7 @@ var highlights={shortest: null, quickest: null};
 var popups={shortest: null, quickest: null};
 var routepoints={shortest: {}, quickest: {}};
 
+
 //
 // Highlight a specific item in the route
 //
