@@ -112,6 +112,9 @@ Only data relevant for routing is displayed and some will therefore be excluded.
 <tr>
 <td><a id="lang_ru_url" href="visualiser.html.ru" title="Страница на русском языке">Русский</a>
 <td>(RU)
+<tr>
+<td><a id="lang_sk_url" href="visualiser.html.sk" title="Stránka v slovenčine">Slovenčina</a>
+<td>(SK)
 </table>
 <a target="translation" href="http://www.routino.org/translations/">Routino Translations</a>
 </div>
