@@ -112,6 +112,9 @@ Only data relevant for routing is displayed and some will therefore be excluded.
 <tr>
 <td><a id="lang_ru_url" href="visualiser.html.ru" title="Страница на русском языке">Русский</a>
 <td>(RU)
+<tr>
+<td><a id="lang_sk_url" href="visualiser.html.sk" title="Stránka v slovenčine">Slovenčina</a>
+<td>(SK)
 </table>
 <a target="translation" href="http://www.routino.org/translations/">Routino Translations</a>
 </div>
@@ -232,10 +235,10 @@ Each segment of the chosen type of highway is drawn.
 <tr><td>Niesklasyfikowana:<td><input name="highway" type="radio" value="unclassified" onchange="displayData('highway');">
 <tr><td>W obszarze zamieszkania: <td><input name="highway" type="radio" value="residential" onchange="displayData('highway');">
 <tr><td>Dojazdowa: <td><input name="highway" type="radio" value="service" onchange="displayData('highway');">
-<tr><td>Track: <td><input name="highway" type="radio" value="track" onchange="displayData('highway');">
+<tr><td>Ścieżka: <td><input name="highway" type="radio" value="track" onchange="displayData('highway');">
 <tr><td>Rowerowa: <td><input name="highway" type="radio" value="cycleway" onchange="displayData('highway');">
 <tr><td>Ścieżka: <td><input name="highway" type="radio" value="path" onchange="displayData('highway');">
-<tr><td>Steps: <td><input name="highway" type="radio" value="steps" onchange="displayData('highway');">
+<tr><td>Kroki: <td><input name="highway" type="radio" value="steps" onchange="displayData('highway');">
 <tr><td>Przeprawa: <td><input name="highway" type="radio" value="ferry" onchange="displayData('highway');">
 </table>
 </form>
@@ -257,7 +260,7 @@ Each segment allowed for the chosen type of transport is drawn.
 <tr><td>Moped: <td><input name="transport" type="radio" value="moped" onchange="displayData('transport');">
 <tr><td>Motocykl:<td><input name="transport" type="radio" value="motorcycle" onchange="displayData('transport');">
 <tr><td>Samochód: <td><input name="transport" type="radio" value="motorcar" onchange="displayData('transport');" checked>
-<tr><td>Goods: <td><input name="transport" type="radio" value="goods" onchange="displayData('transport');">
+<tr><td>Dobra: <td><input name="transport" type="radio" value="goods" onchange="displayData('transport');">
 <tr><td>Pojazd ciężarowy: <td><input name="transport" type="radio" value="hgv" onchange="displayData('transport');">
 <tr><td>Pojazd użyteczności publicznej: <td><input name="transport" type="radio" value="psv" onchange="displayData('transport');">
 </table>
@@ -280,7 +283,7 @@ Each barrier blocking the chosen type of transport is drawn.
 <tr><td>Moped: <td><input name="barrier" type="radio" value="moped" onchange="displayData('barrier');">
 <tr><td>Motocykl:<td><input name="barrier" type="radio" value="motorcycle" onchange="displayData('barrier');">
 <tr><td>Samochód: <td><input name="barrier" type="radio" value="motorcar" onchange="displayData('barrier');" checked>
-<tr><td>Goods: <td><input name="barrier" type="radio" value="goods" onchange="displayData('barrier');">
+<tr><td>Dobra: <td><input name="barrier" type="radio" value="goods" onchange="displayData('barrier');">
 <tr><td>Pojazd ciężarowy: <td><input name="barrier" type="radio" value="hgv" onchange="displayData('barrier');">
 <tr><td>Pojazd użyteczności publicznej: <td><input name="barrier" type="radio" value="psv" onchange="displayData('barrier');">
 </table>
