@@ -3,7 +3,7 @@
 //
 // Part of the Routino routing software.
 //
-// This file Copyright 2008-2014, 2019 Andrew M. Bishop
+// This file Copyright 2008-2014, 2019, 2020 Andrew M. Bishop
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Affero General Public License as published by
@@ -48,9 +48,11 @@ var data_types=[
 
 // Process the URL query string and extract the arguments
 
-var legal={"^lon"  : "^[-0-9.]+$",
-           "^lat"  : "^[-0-9.]+$",
-           "^zoom" : "^[0-9]+$"};
+var legal={"^lon"     : "^[-0-9.]+$",
+           "^lat"     : "^[-0-9.]+$",
+           "^zoom"    : "^[-0-9.]+$",
+           "^data"    : "^.+$",
+           "^subdata" : "^.+$"};
 
 var args={};
 
@@ -83,8 +85,9 @@ if(location.search.length>1)
 ////////////////////////////////////////////////////////////////////////////////
 
 var map;
-var layerMap=[], layerHighlights, layerVectors, layerBoxes;
-var box;
+var layerMap=[], layerHighlights, layerVectors;
+var vectorData=[];
+var displaytype="", displaysubtype="";
 
 //
 // Initialise the 'map' object
@@ -166,18 +169,9 @@ function map_init()             // called from visualiser.html
 
  createPopup();
 
- // Add a boxes layer
-
- layerBoxes = L.rectangle(map.options.maxBounds,{stroke: false, color: "#f00", weight: 1, opacity: 1.0,
-                                                 fill: false});
-
- map.addLayer(layerBoxes);
-
- box=false;
-
  // Move the map
 
- map.on("moveend", updateURLs);
+ map.on("moveend", (function() { displayMoreData(); updateURLs(false);}));
 
  var lon =args["lon"];
  var lat =args["lat"];
@@ -185,6 +179,10 @@ function map_init()             // called from visualiser.html
 
  if(lon !== undefined && lat !== undefined && zoom !== undefined)
    {
+    lat  = Number(lat);
+    lon  = Number(lon);
+    zoom = Number.parseInt(Number(zoom)+0.5);
+
     if(lon<mapprops.westedge) lon=mapprops.westedge;
     if(lon>mapprops.eastedge) lon=mapprops.eastedge;
 
@@ -209,7 +207,17 @@ function map_init()             // called from visualiser.html
     edit_url.href=mapprops.editurl;
    }
 
- updateURLs();
+ // Select the data view if selected
+
+ var datatype=args["data"];
+ var datasubtype=args["subdata"];
+
+ if(datatype !== undefined)
+    displayData(datatype, datasubtype);
+
+ // Update the URL
+
+ updateURLs(false);
 }
 
 
@@ -255,9 +263,19 @@ function buildMapArguments()
 // Update the URLs
 //
 
-function updateURLs()
+function updateURLs(addhistory)
 {
  var mapargs=buildMapArguments();
+ var dataargs=";data=" + displaytype;
+ var libargs=";library=" + mapprops.library;
+
+ if(displaytype === "")
+    dataargs="";
+ else if(displaysubtype !== "")
+    dataargs+=";subdata=" + displaysubtype;
+
+ if(!mapprops.libraries)
+    libargs="";
 
  var links=document.getElementsByTagName("a");
 
@@ -266,17 +284,23 @@ function updateURLs()
     var element=links[i];
 
     if(element.id == "permalink_url")
-       element.href=location.pathname + "?" + mapargs;
+       element.href=location.pathname + "?" + mapargs + dataargs + libargs;
 
     if(element.id == "router_url")
-       element.href="router.html" + "?" + mapargs;
+       if(location.pathname.match(/visualiser\.html\.([a-zA-Z-]+)$/))
+          element.href="router.html" + "." + RegExp.$1 + "?" + mapargs + libargs;
+       else
+          element.href="router.html" + "?" + mapargs + libargs;
 
     if(element.id == "edit_url")
        element.href=mapprops.editurl + "?" + mapargs;
 
     if(element.id.match(/^lang_([a-zA-Z-]+)_url$/))
-       element.href="visualiser.html" + "." + RegExp.$1 + "?" + mapargs;
+       element.href="visualiser.html" + "." + RegExp.$1 + "?" + mapargs + dataargs + libargs;
    }
+
+ if(addhistory)
+    history.replaceState(null, null, location.pathname + "?" + mapargs + dataargs + libargs);
 }
 
 
@@ -424,7 +448,7 @@ function runDumpSuccess(response)
 {
  var string=response.responseText;
 
- if(mapprops.editurl !== undefined && mapprops.editurl !== "")
+ if(mapprops.browseurl !== undefined && mapprops.browseurl !== "")
    {
     var types=["node", "way", "relation"];
     var Types=["Node", "Way", "Relation"];
@@ -536,23 +560,26 @@ function runStatisticsSuccess(response)
 // Get the requested data
 //
 
-function displayData(datatype)  // called from visualiser.html
+function displayData(datatype, datasubtype)  // called from visualiser.html
 {
+ // Display the form entry
+
  for(var data in data_types)
     hideshow_hide(data_types[data]);
 
  if(datatype !== "")
     hideshow_show(datatype);
 
+ if(datasubtype === undefined)
+    datasubtype="";
+
  // Delete the old data
 
+ vectorData=[];
+
  unselectFeature();
 
  layerVectors.clearLayers();
- layerHighlights.clearLayers();
-
- layerBoxes.setStyle({stroke:false});
- box=false;
 
  // Print the status
 
@@ -561,8 +588,84 @@ function displayData(datatype)  // called from visualiser.html
  // Return if just here to clear the data
 
  if(datatype === "")
+   {
+    displaytype = "";
+    displaysubtype = "";
+    updateURLs(true);
     return;
+   }
 
+ // Determine the type of data
+
+ switch(datatype)
+   {
+   case "junctions":
+    break;
+   case "super":
+    break;
+   case "waytype":
+    var waytypes=document.forms["waytypes"].elements["waytype"];
+    for(var w in waytypes)
+       if(datasubtype == waytypes[w].value)
+          waytypes[w].checked=true;
+       else if(waytypes[w].checked)
+          datasubtype=waytypes[w].value;
+    break;
+   case "highway":
+    var highways=document.forms["highways"].elements["highway"];
+    for(var h in highways)
+       if(datasubtype == highways[h].value)
+          highways[h].checked=true;
+       else if(highways[h].checked)
+          datasubtype=highways[h].value;
+    break;
+   case "transport":
+    var transports=document.forms["transports"].elements["transport"];
+    for(var t in transports)
+       if(datasubtype == transports[t].value)
+          transports[t].checked=true;
+       else if(transports[t].checked)
+          datasubtype=transports[t].value;
+    break;
+   case "barrier":
+    var transports=document.forms["barriers"].elements["barrier"];
+    for(var t in transports)
+       if(datasubtype == transports[t].value)
+          transports[t].checked=true;
+       else if(transports[t].checked)
+          datasubtype=transports[t].value;
+    break;
+   case "turns":
+    break;
+   case "speed":
+   case "weight":
+   case "height":
+   case "width":
+   case "length":
+    break;
+   case "property":
+    var properties=document.forms["properties"].elements["property"];
+    for(var p in properties)
+       if(datasubtype == properties[p].value)
+          properties[p].checked=true;
+       else if(properties[p].checked)
+          datasubtype=properties[p].value;
+    break;
+   case "errorlogs":
+    break;
+   }
+
+ // Update the URLs
+
+ displaytype = datatype;
+ displaysubtype = datasubtype;
+
+ displayMoreData();
+}
+
+
+function displayMoreData()
+{
  // Get the new data
 
  var mapbounds=map.getBounds();
@@ -573,11 +676,11 @@ function displayData(datatype)  // called from visualiser.html
  url=url + ";latmin=" + format5f(mapbounds.getSouth());
  url=url + ";lonmax=" + format5f(mapbounds.getEast());
  url=url + ";latmax=" + format5f(mapbounds.getNorth());
- url=url + ";data=" + datatype;
+ url=url + ";data=" + displaytype;
 
  // Use AJAX to get the data
 
- switch(datatype)
+ switch(displaytype)
    {
    case "junctions":
     ajaxGET(url, runJunctionsSuccess, runFailure);
@@ -586,39 +689,19 @@ function displayData(datatype)  // called from visualiser.html
     ajaxGET(url, runSuperSuccess, runFailure);
     break;
    case "waytype":
-    var waytype;
-    var waytypes=document.forms["waytypes"].elements["waytype"];
-    for(var h in waytypes)
-       if(waytypes[h].checked)
-          waytype=waytypes[h].value;
-    url+="-" + waytype;
+    url+="-" + displaysubtype;
     ajaxGET(url, runWaytypeSuccess, runFailure);
     break;
    case "highway":
-    var highway;
-    var highways=document.forms["highways"].elements["highway"];
-    for(var h in highways)
-       if(highways[h].checked)
-          highway=highways[h].value;
-    url+="-" + highway;
+    url+="-" + displaysubtype;
     ajaxGET(url, runHighwaySuccess, runFailure);
     break;
    case "transport":
-    var transport;
-    var transports=document.forms["transports"].elements["transport"];
-    for(var t in transports)
-       if(transports[t].checked)
-          transport=transports[t].value;
-    url+="-" + transport;
+    url+="-" + displaysubtype;
     ajaxGET(url, runTransportSuccess, runFailure);
     break;
    case "barrier":
-    var transport;
-    var transports=document.forms["barriers"].elements["barrier"];
-    for(var t in transports)
-       if(transports[t].checked)
-          transport=transports[t].value;
-    url+="-" + transport;
+    url+="-" + displaysubtype;
     ajaxGET(url, runBarrierSuccess, runFailure);
     break;
    case "turns":
@@ -632,38 +715,15 @@ function displayData(datatype)  // called from visualiser.html
     ajaxGET(url, runLimitSuccess, runFailure);
     break;
    case "property":
-    var property;
-    var properties=document.forms["properties"].elements["property"];
-    for(var p in properties)
-       if(properties[p].checked)
-          property=properties[p].value;
-    url+="-" + property;
+    url+="-" + displaysubtype;
     ajaxGET(url, runPropertySuccess, runFailure);
     break;
    case "errorlogs":
     ajaxGET(url, runErrorlogSuccess, runFailure);
     break;
    }
-}
 
-
-//
-// Add a bounding box
-//
-
-function addBox(words)
-{
- var lat1=words[0];
- var lon1=words[1];
- var lat2=words[2];
- var lon2=words[3];
-
- var bounds = L.latLngBounds(L.latLng(lat1,lon1),L.latLng(lat2,lon2));
-
- layerBoxes.setBounds(bounds);
-
- layerBoxes.setStyle({stroke: true});
- box=true;
+ updateURLs(true);
 }
 
 
@@ -693,12 +753,18 @@ function runJunctionsSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat=words[1];
-       var lon=words[2];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat=Number(words[1]);
+       var lon=Number(words[2]);
        var count=words[3];
 
        var lonlat = L.latLng(lat,lon);
@@ -712,7 +778,7 @@ function runJunctionsSuccess(response)
       }
    }
 
- displayStatus("data","junctions",lines.length-2);
+ displayStatus("data","junctions",Object.keys(vectorData).length);
 }
 
 
@@ -731,12 +797,18 @@ function runSuperSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat=words[1];
-       var lon=words[2];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat=Number(words[1]);
+       var lon=Number(words[2]);
 
        var lonlat = L.latLng(lat,lon);
 
@@ -763,7 +835,7 @@ function runSuperSuccess(response)
       }
    }
 
- displayStatus("data","super",lines.length-2);
+ displayStatus("data","super",Object.keys(vectorData).length);
 }
 
 
@@ -783,14 +855,20 @@ function runWaytypeSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat1=words[1];
-       var lon1=words[2];
-       var lat2=words[3];
-       var lon2=words[4];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat1=Number(words[1]);
+       var lon1=Number(words[2]);
+       var lat2=Number(words[3]);
+       var lon2=Number(words[4]);
 
        var lonlat1 = L.latLng(lat1,lon1);
        var lonlat2 = L.latLng(lat2,lon2);
@@ -823,7 +901,7 @@ function runWaytypeSuccess(response)
       }
    }
 
- displayStatus("data","waytype",lines.length-2);
+ displayStatus("data","waytype",Object.keys(vectorData).length);
 }
 
 
@@ -840,14 +918,20 @@ function runHighwaySuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat1=words[1];
-       var lon1=words[2];
-       var lat2=words[3];
-       var lon2=words[4];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat1=Number(words[1]);
+       var lon1=Number(words[2]);
+       var lat2=Number(words[3]);
+       var lon2=Number(words[4]);
 
        var lonlat1 = L.latLng(lat1,lon1);
        var lonlat2 = L.latLng(lat2,lon2);
@@ -861,7 +945,7 @@ function runHighwaySuccess(response)
       }
    }
 
- displayStatus("data","highway",lines.length-2);
+ displayStatus("data","highway",Object.keys(vectorData).length);
 }
 
 
@@ -878,14 +962,20 @@ function runTransportSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat1=words[1];
-       var lon1=words[2];
-       var lat2=words[3];
-       var lon2=words[4];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat1=Number(words[1]);
+       var lon1=Number(words[2]);
+       var lat2=Number(words[3]);
+       var lon2=Number(words[4]);
 
        var lonlat1 = L.latLng(lat1,lon1);
        var lonlat2 = L.latLng(lat2,lon2);
@@ -899,7 +989,7 @@ function runTransportSuccess(response)
       }
    }
 
- displayStatus("data","transport",lines.length-2);
+ displayStatus("data","transport",Object.keys(vectorData).length);
 }
 
 
@@ -916,12 +1006,18 @@ function runBarrierSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat=words[1];
-       var lon=words[2];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat=Number(words[1]);
+       var lon=Number(words[2]);
 
        var lonlat = L.latLng(lat,lon);
 
@@ -934,7 +1030,7 @@ function runBarrierSuccess(response)
       }
    }
 
- displayStatus("data","barrier",lines.length-2);
+ displayStatus("data","barrier",Object.keys(vectorData).length);
 }
 
 
@@ -951,16 +1047,22 @@ function runTurnsSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat1=words[1];
-       var lon1=words[2];
-       var lat2=words[3];
-       var lon2=words[4];
-       var lat3=words[5];
-       var lon3=words[6];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat1=Number(words[1]);
+       var lon1=Number(words[2]);
+       var lat2=Number(words[3]);
+       var lon2=Number(words[4]);
+       var lat3=Number(words[5]);
+       var lon3=Number(words[6]);
 
        var lonlat1 = L.latLng(lat1,lon1);
        var lonlat2 = L.latLng(lat2,lon2);
@@ -975,7 +1077,7 @@ function runTurnsSuccess(response)
       }
    }
 
- displayStatus("data","turns",lines.length-2);
+ displayStatus("data","turns",Object.keys(vectorData).length);
 }
 
 
@@ -994,12 +1096,18 @@ function runLimitSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat=words[1];
-       var lon=words[2];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat=Number(words[1]);
+       var lon=Number(words[2]);
        var number=words[3];
 
        var lonlat = L.latLng(lat,lon);
@@ -1045,7 +1153,7 @@ function runLimitSuccess(response)
       }
    }
 
- displayStatus("data","limit",lines.length-2);
+ displayStatus("data","limit",Object.keys(vectorData).length);
 }
 
 
@@ -1062,14 +1170,20 @@ function runPropertySuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat1=words[1];
-       var lon1=words[2];
-       var lat2=words[3];
-       var lon2=words[4];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat1=Number(words[1]);
+       var lon1=Number(words[2]);
+       var lat2=Number(words[3]);
+       var lon2=Number(words[4]);
 
        var lonlat1 = L.latLng(lat1,lon1);
        var lonlat2 = L.latLng(lat2,lon2);
@@ -1083,7 +1197,7 @@ function runPropertySuccess(response)
       }
    }
 
- displayStatus("data","property",lines.length-2);
+ displayStatus("data","property",Object.keys(vectorData).length);
 }
 
 
@@ -1100,12 +1214,18 @@ function runErrorlogSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat=words[1];
-       var lon=words[2];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat=Number(words[1]);
+       var lon=Number(words[2]);
 
        var lonlat = L.latLng(lat,lon);
 
@@ -1118,7 +1238,7 @@ function runErrorlogSuccess(response)
       }
    }
 
- displayStatus("data","errorlogs",lines.length-2);
+ displayStatus("data","errorlogs",Object.keys(vectorData).length);
 }
 
 
