@@ -3,7 +3,7 @@
 //
 // Part of the Routino routing software.
 //
-// This file Copyright 2008-2014, 2019 Andrew M. Bishop
+// This file Copyright 2008-2014, 2019, 2020 Andrew M. Bishop
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Affero General Public License as published by
@@ -48,9 +48,11 @@ var data_types=[
 
 // Process the URL query string and extract the arguments
 
-var legal={"^lon"  : "^[-0-9.]+$",
-           "^lat"  : "^[-0-9.]+$",
-           "^zoom" : "^[0-9]+$"};
+var legal={"^lon"     : "^[-0-9.]+$",
+           "^lat"     : "^[-0-9.]+$",
+           "^zoom"    : "^[-0-9.]+$",
+           "^data"    : "^.+$",
+           "^subdata" : "^.+$"};
 
 var args={};
 
@@ -83,10 +85,11 @@ if(location.search.length>1)
 ////////////////////////////////////////////////////////////////////////////////
 
 var map;
-var layerMap=[], layerHighlights, layerVectors, layerBoxes;
+var layerMap=[], layerHighlights, layerVectors;
+var vectorData=[];
 var epsg4326, epsg900913;
-var box;
 var select;
+var displaytype="", displaysubtype="";
 
 //
 // Initialise the 'map' object
@@ -211,16 +214,9 @@ function map_init()             // called from visualiser.html
 
  createPopup();
 
- // Add a boxes layer
-
- layerBoxes = new OpenLayers.Layer.Boxes("Boundary",{displayInLayerSwitcher: false});
- map.addLayer(layerBoxes);
-
- box=null;
-
  // Move the map
 
- map.events.register("moveend", map, updateURLs);
+ map.events.register("moveend", map, (function() { displayMoreData(); updateURLs(false);}));
 
  var lon =args["lon"];
  var lat =args["lat"];
@@ -228,6 +224,10 @@ function map_init()             // called from visualiser.html
 
  if(lon !== undefined && lat !== undefined && zoom !== undefined)
    {
+    lat  = Number(lat);
+    lon  = Number(lon);
+    zoom = Number.parseInt(Number(zoom)+0.5);
+
     if(lon<mapprops.westedge) lon=mapprops.westedge;
     if(lon>mapprops.eastedge) lon=mapprops.eastedge;
 
@@ -244,7 +244,7 @@ function map_init()             // called from visualiser.html
    }
  else
    {
-    map.setCenter(map.restrictedExtent.getCenterLonLat(), map.getZoomForExtent(map.restrictedExtent,true));
+    map.setCenter(map.restrictedExtent.getCenterLonLat(), map.getZoomForExtent(map.restrictedExtent,false));
     map.maxResolution = map.getResolution();
    }
 
@@ -258,7 +258,17 @@ function map_init()             // called from visualiser.html
     edit_url.href=mapprops.editurl;
    }
 
- updateURLs();
+ // Select the data view if selected
+
+ var datatype=args["data"];
+ var datasubtype=args["subdata"];
+
+ if(datatype !== undefined)
+    displayData(datatype, datasubtype);
+
+ // Update the URL
+
+ updateURLs(false);
 }
 
 
@@ -305,9 +315,19 @@ function buildMapArguments()
 // Update the URLs
 //
 
-function updateURLs()
+function updateURLs(addhistory)
 {
  var mapargs=buildMapArguments();
+ var dataargs=";data=" + displaytype;
+ var libargs=";library=" + mapprops.library;
+
+ if(displaytype === "")
+    dataargs="";
+ else if(displaysubtype !== "")
+    dataargs+=";subdata=" + displaysubtype;
+
+ if(!mapprops.libraries)
+    libargs="";
 
  var links=document.getElementsByTagName("a");
 
@@ -316,17 +336,23 @@ function updateURLs()
     var element=links[i];
 
     if(element.id == "permalink_url")
-       element.href=location.pathname + "?" + mapargs;
+       element.href=location.pathname + "?" + mapargs + dataargs + libargs;
 
     if(element.id == "router_url")
-       element.href="router.html" + "?" + mapargs;
+       if(location.pathname.match(/visualiser\.html\.([a-zA-Z-]+)$/))
+          element.href="router.html" + "." + RegExp.$1 + "?" + mapargs + libargs;
+       else
+          element.href="router.html" + "?" + mapargs + libargs;
 
     if(element.id == "edit_url")
        element.href=mapprops.editurl + "?" + mapargs;
 
     if(element.id.match(/^lang_([a-zA-Z-]+)_url$/))
-       element.href="visualiser.html" + "." + RegExp.$1 + "?" + mapargs;
+       element.href="visualiser.html" + "." + RegExp.$1 + "?" + mapargs + dataargs + libargs;
    }
+
+ if(addhistory)
+    history.replaceState(null, null, location.pathname + "?" + mapargs + dataargs + libargs);
 }
 
 
@@ -434,7 +460,7 @@ function runDumpSuccess(response)
 {
  var string=response.responseText;
 
- if(mapprops.editurl !== undefined && mapprops.editurl !== "")
+ if(mapprops.browseurl !== undefined && mapprops.browseurl !== "")
    {
     var types=["node", "way", "relation"];
     var Types=["Node", "Way", "Relation"];
@@ -546,26 +572,28 @@ function runStatisticsSuccess(response)
 // Get the requested data
 //
 
-function displayData(datatype)  // called from visualiser.html
+function displayData(datatype, datasubtype)  // called from visualiser.html
 {
+ // Display the form entry
+
  for(var data in data_types)
     hideshow_hide(data_types[data]);
 
  if(datatype !== "")
     hideshow_show(datatype);
 
+ if(datasubtype === undefined)
+    datasubtype="";
+
  // Delete the old data
 
+ vectorData=[];
+
  unselectFeature();
 
  select.deactivate();
 
  layerVectors.destroyFeatures();
- layerHighlights.destroyFeatures();
-
- if(box !== null)
-    layerBoxes.removeMarker(box);
- box=null;
 
  // Print the status
 
@@ -574,8 +602,84 @@ function displayData(datatype)  // called from visualiser.html
  // Return if just here to clear the data
 
  if(datatype === "")
+   {
+    displaytype = "";
+    displaysubtype = "";
+    updateURLs(true);
     return;
+   }
 
+ // Determine the type of data
+
+ switch(datatype)
+   {
+   case "junctions":
+    break;
+   case "super":
+    break;
+   case "waytype":
+    var waytypes=document.forms["waytypes"].elements["waytype"];
+    for(var w in waytypes)
+       if(datasubtype == waytypes[w].value)
+          waytypes[w].checked=true;
+       else if(waytypes[w].checked)
+          datasubtype=waytypes[w].value;
+    break;
+   case "highway":
+    var highways=document.forms["highways"].elements["highway"];
+    for(var h in highways)
+       if(datasubtype == highways[h].value)
+          highways[h].checked=true;
+       else if(highways[h].checked)
+          datasubtype=highways[h].value;
+    break;
+   case "transport":
+    var transports=document.forms["transports"].elements["transport"];
+    for(var t in transports)
+       if(datasubtype == transports[t].value)
+          transports[t].checked=true;
+       else if(transports[t].checked)
+          datasubtype=transports[t].value;
+    break;
+   case "barrier":
+    var transports=document.forms["barriers"].elements["barrier"];
+    for(var t in transports)
+       if(datasubtype == transports[t].value)
+          transports[t].checked=true;
+       else if(transports[t].checked)
+          datasubtype=transports[t].value;
+    break;
+   case "turns":
+    break;
+   case "speed":
+   case "weight":
+   case "height":
+   case "width":
+   case "length":
+    break;
+   case "property":
+    var properties=document.forms["properties"].elements["property"];
+    for(var p in properties)
+       if(datasubtype == properties[p].value)
+          properties[p].checked=true;
+       else if(properties[p].checked)
+          datasubtype=properties[p].value;
+    break;
+   case "errorlogs":
+    break;
+   }
+
+ // Update the URLs
+
+ displaytype = datatype;
+ displaysubtype = datasubtype;
+
+ displayMoreData();
+}
+
+
+function displayMoreData()
+{
  // Get the new data
 
  var mapbounds=map.getExtent().clone();
@@ -587,11 +691,11 @@ function displayData(datatype)  // called from visualiser.html
  url=url + ";latmin=" + format5f(mapbounds.bottom);
  url=url + ";lonmax=" + format5f(mapbounds.right);
  url=url + ";latmax=" + format5f(mapbounds.top);
- url=url + ";data=" + datatype;
+ url=url + ";data=" + displaytype;
 
  // Use AJAX to get the data
 
- switch(datatype)
+ switch(displaytype)
    {
    case "junctions":
     ajaxGET(url, runJunctionsSuccess, runFailure);
@@ -600,39 +704,19 @@ function displayData(datatype)  // called from visualiser.html
     ajaxGET(url, runSuperSuccess, runFailure);
     break;
    case "waytype":
-    var waytype;
-    var waytypes=document.forms["waytypes"].elements["waytype"];
-    for(var h in waytypes)
-       if(waytypes[h].checked)
-          waytype=waytypes[h].value;
-    url+="-" + waytype;
+    url+="-" + displaysubtype;
     ajaxGET(url, runWaytypeSuccess, runFailure);
     break;
    case "highway":
-    var highway;
-    var highways=document.forms["highways"].elements["highway"];
-    for(var h in highways)
-       if(highways[h].checked)
-          highway=highways[h].value;
-    url+="-" + highway;
+    url+="-" + displaysubtype;
     ajaxGET(url, runHighwaySuccess, runFailure);
     break;
    case "transport":
-    var transport;
-    var transports=document.forms["transports"].elements["transport"];
-    for(var t in transports)
-       if(transports[t].checked)
-          transport=transports[t].value;
-    url+="-" + transport;
+    url+="-" + displaysubtype;
     ajaxGET(url, runTransportSuccess, runFailure);
     break;
    case "barrier":
-    var transport;
-    var transports=document.forms["barriers"].elements["barrier"];
-    for(var t in transports)
-       if(transports[t].checked)
-          transport=transports[t].value;
-    url+="-" + transport;
+    url+="-" + displaysubtype;
     ajaxGET(url, runBarrierSuccess, runFailure);
     break;
    case "turns":
@@ -646,37 +730,15 @@ function displayData(datatype)  // called from visualiser.html
     ajaxGET(url, runLimitSuccess, runFailure);
     break;
    case "property":
-    var property;
-    var properties=document.forms["properties"].elements["property"];
-    for(var p in properties)
-       if(properties[p].checked)
-          property=properties[p].value;
-    url+="-" + property;
+    url+="-" + displaysubtype;
     ajaxGET(url, runPropertySuccess, runFailure);
     break;
    case "errorlogs":
     ajaxGET(url, runErrorlogSuccess, runFailure);
     break;
    }
-}
 
-
-//
-// Add a bounding box
-//
-
-function addBox(words)
-{
- var lat1=words[0];
- var lon1=words[1];
- var lat2=words[2];
- var lon2=words[3];
-
- var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
-
- box = new OpenLayers.Marker.Box(bounds);
-
- layerBoxes.addMarker(box);
+ updateURLs(true);
 }
 
 
@@ -715,12 +777,18 @@ function runJunctionsSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat=words[1];
-       var lon=words[2];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat=Number(words[1]);
+       var lon=Number(words[2]);
        var count=words[3];
 
        var lonlat= new OpenLayers.LonLat(lon,lat).transform(epsg4326,epsg900913);
@@ -735,7 +803,7 @@ function runJunctionsSuccess(response)
 
  layerVectors.addFeatures(features);
 
- displayStatus("data","junctions",lines.length-2);
+ displayStatus("data","junctions",Object.keys(vectorData).length);
 }
 
 
@@ -764,12 +832,18 @@ function runSuperSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat=words[1];
-       var lon=words[2];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat=Number(words[1]);
+       var lon=Number(words[2]);
 
        var lonlat= new OpenLayers.LonLat(lon,lat).transform(epsg4326,epsg900913);
 
@@ -794,7 +868,7 @@ function runSuperSuccess(response)
 
  layerVectors.addFeatures(features);
 
- displayStatus("data","super",lines.length-2);
+ displayStatus("data","super",Object.keys(vectorData).length);
 }
 
 
@@ -816,14 +890,20 @@ function runWaytypeSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat1=words[1];
-       var lon1=words[2];
-       var lat2=words[3];
-       var lon2=words[4];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat1=Number(words[1]);
+       var lon1=Number(words[2]);
+       var lat2=Number(words[3]);
+       var lon2=Number(words[4]);
 
        var lonlat1 = new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
        var lonlat2 = new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
@@ -856,7 +936,7 @@ function runWaytypeSuccess(response)
 
  layerVectors.addFeatures(features);
 
- displayStatus("data","waytype",lines.length-2);
+ displayStatus("data","waytype",Object.keys(vectorData).length);
 }
 
 
@@ -879,14 +959,20 @@ function runHighwaySuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat1=words[1];
-       var lon1=words[2];
-       var lat2=words[3];
-       var lon2=words[4];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat1=Number(words[1]);
+       var lon1=Number(words[2]);
+       var lat2=Number(words[3]);
+       var lon2=Number(words[4]);
 
        var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
        var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
@@ -904,7 +990,7 @@ function runHighwaySuccess(response)
 
  layerVectors.addFeatures(features);
 
- displayStatus("data","highway",lines.length-2);
+ displayStatus("data","highway",Object.keys(vectorData).length);
 }
 
 
@@ -927,14 +1013,20 @@ function runTransportSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat1=words[1];
-       var lon1=words[2];
-       var lat2=words[3];
-       var lon2=words[4];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat1=Number(words[1]);
+       var lon1=Number(words[2]);
+       var lat2=Number(words[3]);
+       var lon2=Number(words[4]);
 
        var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
        var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
@@ -952,7 +1044,7 @@ function runTransportSuccess(response)
 
  layerVectors.addFeatures(features);
 
- displayStatus("data","transport",lines.length-2);
+ displayStatus("data","transport",Object.keys(vectorData).length);
 }
 
 
@@ -975,12 +1067,18 @@ function runBarrierSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat=words[1];
-       var lon=words[2];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat=Number(words[1]);
+       var lon=Number(words[2]);
 
        var lonlat= new OpenLayers.LonLat(lon,lat).transform(epsg4326,epsg900913);
 
@@ -994,7 +1092,7 @@ function runBarrierSuccess(response)
 
  layerVectors.addFeatures(features);
 
- displayStatus("data","barrier",lines.length-2);
+ displayStatus("data","barrier",Object.keys(vectorData).length);
 }
 
 
@@ -1017,16 +1115,22 @@ function runTurnsSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat1=words[1];
-       var lon1=words[2];
-       var lat2=words[3];
-       var lon2=words[4];
-       var lat3=words[5];
-       var lon3=words[6];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat1=Number(words[1]);
+       var lon1=Number(words[2]);
+       var lat2=Number(words[3]);
+       var lon2=Number(words[4]);
+       var lat3=Number(words[5]);
+       var lon3=Number(words[6]);
 
        var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
        var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
@@ -1046,7 +1150,7 @@ function runTurnsSuccess(response)
 
  layerVectors.addFeatures(features);
 
- displayStatus("data","turns",lines.length-2);
+ displayStatus("data","turns",Object.keys(vectorData).length);
 }
 
 
@@ -1076,12 +1180,18 @@ function runLimitSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat=words[1];
-       var lon=words[2];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat=Number(words[1]);
+       var lon=Number(words[2]);
        var number=words[3];
 
        var lonlat= new OpenLayers.LonLat(lon,lat).transform(epsg4326,epsg900913);
@@ -1120,7 +1230,7 @@ function runLimitSuccess(response)
 
  layerVectors.addFeatures(features);
 
- displayStatus("data","limit",lines.length-2);
+ displayStatus("data","limit",Object.keys(vectorData).length);
 }
 
 
@@ -1143,14 +1253,20 @@ function runPropertySuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat1=words[1];
-       var lon1=words[2];
-       var lat2=words[3];
-       var lon2=words[4];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat1=Number(words[1]);
+       var lon1=Number(words[2]);
+       var lat2=Number(words[3]);
+       var lon2=Number(words[4]);
 
        var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
        var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
@@ -1168,7 +1284,7 @@ function runPropertySuccess(response)
 
  layerVectors.addFeatures(features);
 
- displayStatus("data","property",lines.length-2);
+ displayStatus("data","property",Object.keys(vectorData).length);
 }
 
 
@@ -1191,12 +1307,18 @@ function runErrorlogSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
-       var lat=words[1];
-       var lon=words[2];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
+       var lat=Number(words[1]);
+       var lon=Number(words[2]);
 
        var lonlat = new OpenLayers.LonLat(lon,lat).transform(epsg4326,epsg900913);
 
@@ -1210,7 +1332,7 @@ function runErrorlogSuccess(response)
 
  layerVectors.addFeatures(features);
 
- displayStatus("data","errorlogs",lines.length-2);
+ displayStatus("data","errorlogs",Object.keys(vectorData).length);
 }
 
 
